@@ -462,7 +462,7 @@ int GSInterpreterLib::presize(void * /* device*/, int width, int height,
 }
 
 int GSInterpreterLib::size(void * /* device*/, int /*width*/, int /*height*/,
-        int raster, unsigned int /*format*/, unsigned char *pimage)
+        int /*raster*/, unsigned int /*format*/, unsigned char *pimage)
 {
     m_wasSize=true;
     m_imageChar=pimage;
