@@ -11,13 +11,9 @@
 #ifndef _OKULAR_GENERATOR_H_
 #define _OKULAR_GENERATOR_H_
 
-#include "okular_export.h"
-
-#define OKULAR_EXPORT_PLUGIN( classname ) \
-    extern "C" { \
-         OKULAR_EXPORT Okular::Generator* create_plugin() { return new classname(); } \
-    }
-
+#include <okular/core/okular_export.h>
+#include <okular/core/global.h>
+#include <core/pagesize.h>
 
 #include <QtCore/QList>
 #include <QtCore/QObject>
@@ -27,22 +23,29 @@
 
 #include <kmimetype.h>
 
-#include "document.h"
+#define OKULAR_EXPORT_PLUGIN( classname ) \
+    extern "C" { \
+         OKULAR_EXPORT Okular::Generator* create_plugin() { return new classname(); } \
+    }
 
-class KConfigDialog;
 class KIcon;
 class KPrinter;
 class kdbgstream;
 
 namespace Okular {
 
+class Document;
+class DocumentFonts;
+class DocumentInfo;
+class DocumentSynopsis;
+class EmbeddedFile;
 class Link;
 class Page;
 class PixmapRequest;
 class TextPage;
 
-/* Note: on contents generation and asyncronous queries.
- * Many observers may want to request data syncronously or asyncronously.
+/* Note: on contents generation and asynchronous queries.
+ * Many observers may want to request data syncronously or asynchronously.
  * - Sync requests. These should be done in-place.
  * - Async request must be done in real background. That usually means a
  *   thread, such as QThread derived classes.
@@ -55,8 +58,8 @@ class TextPage;
  * @short Defines an entry for the export menu
  *
  * This class encapsulates information about an export format.
- * Every Generator can support 0 ore more export formats which can be
- * queried with @see Generator::exportFormats().
+ * Every Generator can support 0 or more export formats which can be
+ * queried with @ref Generator::exportFormats().
  */
 class OKULAR_EXPORT ExportFormat
 {
@@ -90,7 +93,14 @@ class OKULAR_EXPORT ExportFormat
          */
         ~ExportFormat();
 
+        /**
+         * @internal
+         */
         ExportFormat( const ExportFormat &other );
+
+        /**
+         * @internal
+         */
         ExportFormat& operator=( const ExportFormat &other );
 
         /**
@@ -116,18 +126,39 @@ class OKULAR_EXPORT ExportFormat
 /**
  * @short [Abstract Class] The information generator.
  *
- * Most of class members are pure virtuals and they must be implemented to
- * provide a class that builds contents (graphics and text).
+ * Most of class members are virtuals and some of them pure virtual. The pure
+ * virtuals provide the minimal functionalities for a Generator, that is being
+ * able to generate QPixmap for the Page 's of the Document.
  *
- * Generation/query is requested by the 'Document' class only, and that
- * class stores the resulting data into 'Page's. The data will then be
- * displayed by the GUI components (pageView, thumbnailList, etc..).
+ * Implementing the other functions will make the Generator able to provide
+ * more contents and/or functionalities (like text extraction).
+ *
+ * Generation/query is requested by the Document class only, and that
+ * class stores the resulting data into Page s. The data will then be
+ * displayed by the GUI components (PageView, ThumbnailList, etc..).
+ *
+ * @see PrintInterface, ConfigInterface, GuiInterface
  */
 class OKULAR_EXPORT Generator : public QObject
 {
+    friend class PixmapGenerationThread;
+    friend class TextPageGenerationThread;
+
     Q_OBJECT
 
     public:
+        /**
+         * Describe the possible optional features that a Generator can
+         * provide.
+         */
+        enum GeneratorFeature
+        {
+            Threaded,
+            TextExtraction,    ///< Whether the Generator can extract text from the document in the form of TextPage's
+            ReadRawData,       ///< Whether the Generator can read a document directly from its raw data.
+            PageSizes          ///< Whether the Generator can change the size of the document pages.
+        };
+
         /**
          * Creates a new generator.
          */
@@ -151,6 +182,16 @@ class OKULAR_EXPORT Generator : public QObject
          */
         virtual bool loadDocument( const QString & fileName, QVector< Page * > & pagesVector ) = 0;
 
+        /**
+         * Loads the document from the raw data @p fileData and fills the
+         * @p pagesVector with the parsed pages.
+         *
+         * @note the Generator has to have the feature @ref ReadRawData enabled
+         *
+         * @returns true on success, false otherwise.
+         */
+        virtual bool loadDocumentFromData( const QByteArray & fileData, QVector< Page * > & pagesVector );
+
         /**
          * This method is called when the document is closed and not used
          * any longer.
@@ -160,32 +201,34 @@ class OKULAR_EXPORT Generator : public QObject
         virtual bool closeDocument() = 0;
 
         /**
-         * This method returns whether the generator can create pixmaps for
-         * each page in a synchronous or asynchronous way, depending on @p async.
+         * This method returns whether the generator is ready to
+         * handle a new pixmap request.
          */
-        virtual bool canGeneratePixmap( bool async ) const = 0;
+        virtual bool canGeneratePixmap() const;
 
         /**
-         * This method is called to create a pixmap for a page. The page number,
-         * width and height is encapsulated in the page @p request.
+         * This method can be called to trigger the generation of
+         * a new pixmap as described by @p request.
          */
-        virtual void generatePixmap( PixmapRequest * request ) = 0;
+        virtual void generatePixmap( PixmapRequest * request );
 
         /**
-         * This method returns whether the generator can create text pages,
-         * which are used for search and text extraction.
-         *
-         * @returns false as default.
+         * This method returns whether the generator is ready to
+         * handle a new text page request.
          */
         virtual bool canGenerateTextPage() const;
 
         /**
-         * This method is called to create a so called 'text page' for the given @p page.
+         * This method can be called to trigger the generation of
+         * a text page for the given @p page.
          *
-         * A text page is an abstract description of the readable text of the page.
-         * It's used for search and text extraction.
+         * The generation is done synchronous or asynchronous, depending
+         * on the @p type parameter and the capabilities of the
+         * generator (e.g. multithreading).
+         *
+         * @see TextPage
          */
-        virtual void generateSyncTextPage( Page *page );
+        virtual void generateTextPage( Page * page, enum GenerationType type = Synchronous );
 
         /**
          * Returns the general information object of the document or 0 if
@@ -213,67 +256,38 @@ class OKULAR_EXPORT Generator : public QObject
 
         /**
          * This enum identifies the metric of the page size.
-         *
-         * @li None - page size is not defined in a physical metric
-         * @li Points - page size is given in 1/72 inches
          */
         enum PageSizeMetric
         {
-          None,
-          Points
+          None,   ///< The page size is not defined in a physical metric.
+          Points  ///< The page size is given in 1/72 inches.
         };
 
         /**
-         * This method returns the metric of the page size. Default is @see None.
+         * This method returns the metric of the page size. Default is @ref None.
          */
         virtual PageSizeMetric pagesSizeMetric() const;
 
         /**
-         * This method returns whether given action (@see Document::Permission) is
+         * This method returns whether given action (@ref Permission) is
          * allowed in this document.
          */
-        virtual bool isAllowed( int action ) const;
-
-        // gui stuff
-        virtual QString getXMLFile() const;
-        virtual void setupGUI( KActionCollection*, QToolBox* );
-        virtual void freeGUI();
-
-        /**
-         * This method returns whether the generator supports searching. Default is false.
-         */
-        virtual bool supportsSearching() const;
-
-        /**
-         * Returns whether the generator supports rotation of the pages. Default is false.
-         */
-        virtual bool supportsRotation() const;
+        virtual bool isAllowed( Permissions action ) const;
 
         /**
          * This method is called when the orientation has been changed by the user.
          */
-        virtual void rotationChanged( int orientation, int oldOrientation );
+        virtual void rotationChanged( Rotation orientation, Rotation oldOrientation );
 
         /**
-         * Returns whether the generator supports paper sizes. Default is false.
+         * Returns the list of supported page sizes.
          */
-        virtual bool supportsPaperSizes() const;
+        virtual PageSize::List pageSizes() const;
 
         /**
-         * Returns the list of supported paper sizes.
+         * This method is called when the page size has been changed by the user.
          */
-        virtual QStringList paperSizes() const;
-
-        /**
-         * This method is called to set the paper size of the given @p pages
-         * to the given @p paperSize.
-         */
-        virtual void setPaperSize( QVector<Page*> &pages, int paperSize );
-
-        /**
-         * Returns true if the generator configures the printer itself, false otherwise.
-         */
-        virtual bool canConfigurePrinter() const;
+        virtual void pageSizeChanged( const PageSize &pageSize, const PageSize &oldPageSize );
 
         /**
          * This method is called to print the document to the given @p printer.
@@ -286,19 +300,6 @@ class OKULAR_EXPORT Generator : public QObject
          */
         virtual QVariant metaData( const QString &key, const QVariant &option ) const;
 
-        /**
-         * This method is called to tell the generator to re-parse its configuration.
-         *
-         * Returns true if something has changed.
-         */
-        virtual bool reparseConfig();
-
-        /**
-         * This method allows the generator to add custom configuration pages to the
-         * config @p dialog of okular.
-         */
-        virtual void addPages( KConfigDialog *dialog );
-
         /**
          * Returns the list of additional supported export formats.
          */
@@ -310,9 +311,14 @@ class OKULAR_EXPORT Generator : public QObject
          */
         virtual bool exportTo( const QString &fileName, const ExportFormat &format );
 
+        /**
+         * Query for the specified @p feature.
+         */
+        bool hasFeature( GeneratorFeature feature ) const;
+
     Q_SIGNALS:
         /**
-         * This signal should be emitted whenever an error occured in the generator.
+         * This signal should be emitted whenever an error occurred in the generator.
          *
          * @param message The message which should be shown to the user.
          * @param duration The time that the message should be shown to the user.
@@ -337,19 +343,44 @@ class OKULAR_EXPORT Generator : public QObject
 
     protected:
         /**
-         * This method must be called when the pixmap request triggered by @see generatePixmap()
+         * This method must be called when the pixmap request triggered by generatePixmap()
          * has been finished.
          */
-        void signalRequestDone( PixmapRequest * request );
+        void signalPixmapRequestDone( PixmapRequest * request );
+
+        /**
+         * Returns the image of the page as specified in
+         * the passed pixmap @p request.
+         *
+         * @warning this method may be executed in its own separated thread if the
+         * @ref Threaded is enabled!
+         */
+        virtual QImage image( PixmapRequest *page );
+
+        /**
+         * Returns the text page for the given @p page.
+         *
+         * @warning this method may be executed in its own separated thread if the
+         * @ref Threaded is enabled!
+         */
+        virtual TextPage* textPage( Page *page );
 
         /**
          * Returns a pointer to the document.
          */
         Document * document() const;
 
+        /**
+         * Toggle the @p feature .
+         */
+        void setFeature( GeneratorFeature feature, bool on = true );
+
     private:
         class Private;
         Private* const d;
+
+        Q_PRIVATE_SLOT( d, void pixmapGenerationFinished() )
+        Q_PRIVATE_SLOT( d, void textpageGenerationFinished() )
 };
 
 /**
@@ -417,6 +448,11 @@ class OKULAR_EXPORT PixmapRequest
          */
         Page *page() const;
 
+        /**
+         * Internal usage.
+         */
+        void swap();
+
     protected:
         /**
          * Internal usage.
