@@ -10,35 +10,44 @@
 #ifndef _DJVU_GENERATOR_H_
 #define _DJVU_GENERATOR_H_
 
-#include "core/generator.h"
+#include <okular/core/generator.h>
+
 #include <qvector.h>
 
-class KDjVu;
+#include "kdjvu.h"
+
+namespace Okular {
+class Annotation;
+class ObjectRect;
+}
 
 class DjVuGenerator : public Okular::Generator
 {
     Q_OBJECT
     public:
         DjVuGenerator();
+        ~DjVuGenerator();
         bool loadDocument( const QString & fileName, QVector<Okular::Page*> & pagesVector );
         bool closeDocument();
 
         // pixmap generation
-        bool canGeneratePixmap( bool async ) const;
+        bool canGeneratePixmap() const;
         void generatePixmap( Okular::PixmapRequest * request );
 
         // document information
         const Okular::DocumentInfo * generateDocumentInfo();
         const Okular::DocumentSynopsis * generateDocumentSynopsis();
 
-        // rotation handling
-        bool supportsRotation() const { return true; };
+        // printing
+        bool print( KPrinter& printer );
 
     private slots:
         void djvuImageGenerated( int page, const QImage & img );
 
     private:
         void loadPages( QVector<Okular::Page*> & pagesVector, int rotation );
+        Okular::ObjectRect* convertKDjVuLink( int page, KDjVu::Link * link ) const;
+        Okular::Annotation* convertKDjVuAnnotation( int w, int h, KDjVu::Annotation * ann ) const;
 
         KDjVu *m_djvu;
 
