@@ -12,8 +12,8 @@
 #include <qbytearray.h>
 #include <qdom.h>
 #include <qfile.h>
+#include <qhash.h>
 #include <qlist.h>
-#include <qmap.h>
 #include <qpainter.h>
 #include <qstring.h>
 
@@ -23,6 +23,8 @@
 #include <libdjvu/ddjvuapi.h>
 #include <libdjvu/miniexp.h>
 
+#include <stdio.h>
+
 kdbgstream &operator<<( kdbgstream & s, const ddjvu_rect_t &r )
 {
     s << "[" << r.x << "," << r.y << " - " << r.w << "x" << r.h << "]";
@@ -46,6 +48,9 @@ static void which_ddjvu_message( const ddjvu_message_t *msg )
         case DDJVU_CHUNK:
             kDebug() << "CHUNK: '" << QByteArray( msg->m_chunk.chunkid ) << "'" << endl;
             break;
+        case DDJVU_PROGRESS:
+            kDebug() << "PROGRESS: '" << msg->m_progress.percent << "'" << endl;
+            break;
         default: ;
     }
 #else
@@ -195,6 +200,75 @@ QString KDjVu::UrlLink::url() const
     return m_url;
 }
 
+// KDjVu::Annotation
+
+KDjVu::Annotation::~Annotation()
+{
+}
+
+QPoint KDjVu::Annotation::point() const
+{
+    return m_point;
+}
+
+QString KDjVu::Annotation::comment() const
+{
+    return m_comment;
+}
+
+QColor KDjVu::Annotation::color() const
+{
+    return m_color;
+}
+
+// KDjVu::TextAnnotation
+
+KDjVu::TextAnnotation::TextAnnotation()
+  : m_inlineText( true )
+{
+}
+
+QSize KDjVu::TextAnnotation::size() const
+{
+    return m_size;
+}
+
+int KDjVu::TextAnnotation::type() const
+{
+    return KDjVu::Annotation::TextAnnotation;
+}
+
+bool KDjVu::TextAnnotation::inlineText() const
+{
+    return m_inlineText;
+}
+
+// KDjVu::LineAnnotation
+
+KDjVu::LineAnnotation::LineAnnotation()
+  : m_isArrow( false ), m_width( 1 )
+{
+}
+
+int KDjVu::LineAnnotation::type() const
+{
+    return KDjVu::Annotation::LineAnnotation;
+}
+
+QPoint KDjVu::LineAnnotation::point2() const
+{
+    return m_point2;
+}
+
+bool KDjVu::LineAnnotation::isArrow() const
+{
+    return m_isArrow;
+}
+
+int KDjVu::LineAnnotation::width() const
+{
+    return m_width;
+}
 
 
 
@@ -213,16 +287,19 @@ class KDjVu::Private
         void fillBookmarksRecurse( QDomDocument& maindoc, QDomNode& curnode,
             miniexp_t exp, int offset = -1 );
 
+        void readMetaData( int page );
+
         ddjvu_context_t *m_djvu_cxt;
         ddjvu_document_t *m_djvu_document;
         ddjvu_format_t *m_format;
+        unsigned int* m_formatmask;
 
         QVector<KDjVu::Page*> m_pages;
         QVector<ddjvu_page_t *> m_pages_cache;
 
         QList<ImageCacheItem*> mImgCache;
 
-        QMap<QString, QString> m_metaData;
+        QHash<QString, QVariant> m_metaData;
         QDomDocument * m_docBookmarks;
 };
 
@@ -317,17 +394,47 @@ void KDjVu::Private::fillBookmarksRecurse( QDomDocument& maindoc, QDomNode& curn
     }
 }
 
+void KDjVu::Private::readMetaData( int page )
+{
+    if ( !m_djvu_document )
+        return;
+
+    miniexp_t annots;
+    while ( ( annots = ddjvu_document_get_pageanno( m_djvu_document, page ) ) == miniexp_dummy )
+        handle_ddjvu_messages( m_djvu_cxt, true );
+
+    if ( !miniexp_listp( annots ) || miniexp_length( annots ) == 0 )
+        return;
+
+    miniexp_t exp = miniexp_nth( 0, annots );
+    int size = miniexp_length( exp );
+    if ( size <= 1 ||
+         qstrncmp( miniexp_to_name( miniexp_nth( 0, exp ) ), "metadata", 8 ) )
+        return;
+
+    for ( int i = 1; i < size; ++i )
+    {
+        miniexp_t cur = miniexp_nth( i, exp );
+        if ( miniexp_length( cur ) != 2 )
+            continue;
+
+        QString id = QString::fromUtf8( miniexp_to_name( miniexp_nth( 0, cur ) ) );
+        QString value = QString::fromUtf8( miniexp_to_str( miniexp_nth( 1, cur ) ) );
+        m_metaData[ id.toLower() ] = value;
+    }
+}
+
 
 KDjVu::KDjVu() : QObject(), d( new Private )
 {
     // creating the djvu context
     d->m_djvu_cxt = ddjvu_context_create( "KDjVu" );
     // creating the rendering format
-    unsigned int* mask = new unsigned int[3];
-    mask[0] = 0x00ff0000;
-    mask[1] = 0x0000ff00;
-    mask[2] = 0x000000ff;
-    d->m_format = ddjvu_format_create( DDJVU_FORMAT_RGBMASK32, 3, mask );
+    d->m_formatmask = new unsigned int[3];
+    d->m_formatmask[0] = 0x00ff0000;
+    d->m_formatmask[1] = 0x0000ff00;
+    d->m_formatmask[2] = 0x000000ff;
+    d->m_format = ddjvu_format_create( DDJVU_FORMAT_RGBMASK32, 3, d->m_formatmask );
     ddjvu_format_set_row_order( d->m_format, 1 );
     ddjvu_format_set_y_direction( d->m_format, 1 );
 }
@@ -338,6 +445,7 @@ KDjVu::~KDjVu()
     closeFile();
 
     ddjvu_format_release( d->m_format );
+    delete [] d->m_formatmask;
     ddjvu_context_release( d->m_djvu_cxt );
 
     delete d;
@@ -388,31 +496,37 @@ bool KDjVu::openFile( const QString & fileName )
     if ( !doctype.isEmpty() )
         d->m_metaData[ "documentType" ] = doctype;
     // get the number of components
-    d->m_metaData[ "componentFile" ] = QString::number( ddjvu_document_get_filenum( d->m_djvu_document ) );
+    d->m_metaData[ "componentFile" ] = ddjvu_document_get_filenum( d->m_djvu_document );
 
     // read the pages
     for ( int i = 0; i < numofpages; ++i )
     {
-        // wait for the new page to be loaded
-        ddjvu_page_t *newpage = ddjvu_page_create_by_pageno( d->m_djvu_document, i );
         ddjvu_status_t sts;
-        while ( ( sts = ddjvu_page_decoding_status( newpage ) ) < DDJVU_JOB_OK )
+        ddjvu_pageinfo_t info;
+        while ( ( sts = ddjvu_document_get_pageinfo( d->m_djvu_document, i, &info ) ) < DDJVU_JOB_OK )
             handle_ddjvu_messages( d->m_djvu_cxt, true );
         if ( sts >= DDJVU_JOB_FAILED )
         {
             kDebug() << "\t>>> page " << i << " failed: " << sts << endl;
-            break;
+            return false;
         }
-        d->m_pages_cache[i] = newpage;
 
         KDjVu::Page *p = new KDjVu::Page();
-        p->m_width = ddjvu_page_get_width( newpage );
-        p->m_height = ddjvu_page_get_height( newpage );
-        p->m_dpi = ddjvu_page_get_resolution( newpage );
-        p->m_orientation = flipRotation( ddjvu_page_get_initial_rotation( newpage ) );
+        p->m_width = info.width;
+        p->m_height = info.height;
+        p->m_dpi = info.dpi;
+#if DDJVUAPI_VERSION >= 18
+        p->m_orientation = flipRotation( info.rotation );
+#else
+        p->m_orientation = 0;
+#endif
         d->m_pages[i] = p;
     }
 
+    // reading the metadata from the first page only should be enough
+    if ( numofpages > 0 )
+        d->readMetaData( 0 );
+
     return true;
 }
 
@@ -423,6 +537,7 @@ void KDjVu::closeFile()
     d->m_docBookmarks = 0;
     // deleting the pages
     qDeleteAll( d->m_pages );
+    d->m_pages.clear();
     // releasing the djvu pages
     QVector<ddjvu_page_t *>::Iterator it = d->m_pages_cache.begin(), itEnd = d->m_pages_cache.end();
     for ( ; it != itEnd; ++it )
@@ -430,6 +545,7 @@ void KDjVu::closeFile()
     d->m_pages_cache.clear();
     // clearing the image cache
     qDeleteAll( d->mImgCache );
+    d->mImgCache.clear();
     // clearing the old metadata
     d->m_metaData.clear();
     // releasing the old document
@@ -438,9 +554,10 @@ void KDjVu::closeFile()
     d->m_djvu_document = 0;
 }
 
-QString KDjVu::metaData( const QString & key ) const
+QVariant KDjVu::metaData( const QString & key ) const
 {
-    return d->m_metaData.contains( key ) ? d->m_metaData[ key ] : QString();
+    QHash<QString, QVariant>::ConstIterator it = d->m_metaData.find( key );
+    return it != d->m_metaData.end() ? it.value() : QVariant();
 }
 
 const QDomDocument * KDjVu::documentBookmarks() const
@@ -450,34 +567,54 @@ const QDomDocument * KDjVu::documentBookmarks() const
     return d->m_docBookmarks;
 }
 
-QList<KDjVu::Link*> KDjVu::linksForPage( int pageNum ) const
+void KDjVu::linksAndAnnotationsForPage( int pageNum, QList<KDjVu::Link*> *links, QList<KDjVu::Annotation*> *annotations ) const
 {
-    if ( ( pageNum < 0 ) || ( pageNum >= d->m_pages.count() ) )
-        return QList<KDjVu::Link*>();
+    if ( ( pageNum < 0 ) || ( pageNum >= d->m_pages.count() ) || ( !links && !annotations ) )
+        return;
 
     miniexp_t annots;
     while ( ( annots = ddjvu_document_get_pageanno( d->m_djvu_document, pageNum ) ) == miniexp_dummy )
         handle_ddjvu_messages( d->m_djvu_cxt, true );
 
     if ( !miniexp_listp( annots ) )
-        return QList<KDjVu::Link*>();
+        return;
 
-    QList<KDjVu::Link*> ret;
+    if ( links )
+        links->clear();
+    if ( annotations )
+        annotations->clear();
 
     int l = miniexp_length( annots );
     for ( int i = 0; i < l; ++i )
     {
         miniexp_t cur = miniexp_nth( i, annots );
         int num = miniexp_length( cur );
-        if ( ( num <= 0 ) || !miniexp_symbolp( miniexp_nth( 0, cur ) ) ||
+        if ( ( num < 4 ) || !miniexp_symbolp( miniexp_nth( 0, cur ) ) ||
              ( qstrncmp( miniexp_to_name( miniexp_nth( 0, cur ) ), "maparea", 7 ) != 0 ) )
             continue;
 
         QString target;
+        QString type;
+        if ( miniexp_symbolp( miniexp_nth( 0, miniexp_nth( 3, cur ) ) ) )
+            type = QString::fromUtf8( miniexp_to_name( miniexp_nth( 0, miniexp_nth( 3, cur ) ) ) );
         KDjVu::Link* link = 0;
-        if ( miniexp_stringp( miniexp_nth( 1, cur ) ) )
+        KDjVu::Annotation* ann = 0;
+        miniexp_t urlexp = miniexp_nth( 1, cur );
+        if ( links &&
+             ( type == QLatin1String( "rect" ) ||
+               type == QLatin1String( "oval" ) ||
+               type == QLatin1String( "poly" ) ) )
         {
-            QString target = QString::fromUtf8( miniexp_to_str( miniexp_nth( 1, cur ) ) );
+            if ( miniexp_stringp( urlexp ) )
+            {
+                target = QString::fromUtf8( miniexp_to_str( miniexp_nth( 1, cur ) ) );
+            }
+            else if ( miniexp_listp( urlexp ) && ( miniexp_length( urlexp ) == 3 ) &&
+                      miniexp_symbolp( miniexp_nth( 0, urlexp ) ) &&
+                      ( qstrncmp( miniexp_to_name( miniexp_nth( 0, urlexp ) ), "url", 3 ) == 0 ) )
+            {
+                target = QString::fromUtf8( miniexp_to_str( miniexp_nth( 1, urlexp ) ) );
+            }
             if ( target.isEmpty() || ( ( target.length() > 0 ) && target.at(0) == QLatin1Char( '#' ) ) )
             {
                 KDjVu::PageLink* plink = new KDjVu::PageLink();
@@ -491,22 +628,40 @@ QList<KDjVu::Link*> KDjVu::linksForPage( int pageNum ) const
                 link = ulink;
             }
         }
-        else
+        else if ( annotations &&
+                  ( type == QLatin1String( "text" ) ||
+                    type == QLatin1String( "line" ) ) )
         {
-            // TODO: external hyperlink
+            miniexp_t area = miniexp_nth( 3, cur );
+            int a = miniexp_to_int( miniexp_nth( 1, area ) );
+            int b = miniexp_to_int( miniexp_nth( 2, area ) );
+            int c = miniexp_to_int( miniexp_nth( 3, area ) );
+            int d = miniexp_to_int( miniexp_nth( 4, area ) );
+            if ( type == QLatin1String( "text" ) )
+            {
+                KDjVu::TextAnnotation * textann = new KDjVu::TextAnnotation();
+                textann->m_size = QSize( c, d );
+                ann = textann;
+            }
+            else if ( type == QLatin1String( "line" ) )
+            {
+                KDjVu::LineAnnotation * lineann = new KDjVu::LineAnnotation();
+                lineann->m_point2 = QPoint( c, d );
+                ann = lineann;
+            }
+            ann->m_point = QPoint( a, b );
+            ann->m_comment = QString::fromUtf8( miniexp_to_str( miniexp_nth( 2, cur ) ) );
         }
-        if ( link )
+        if ( link /* safety check */ && links )
         {
             link->m_area = KDjVu::Link::UnknownArea;
             miniexp_t area = miniexp_nth( 3, cur );
             int arealength = miniexp_length( area );
-            if ( ( arealength == 5 ) && miniexp_symbolp( miniexp_nth( 0, area ) ) &&
-                 ( ( qstrncmp( miniexp_to_name( miniexp_nth( 0, area ) ), "rect", 4 ) == 0 ) ||
-                   ( qstrncmp( miniexp_to_name( miniexp_nth( 0, area ) ), "oval", 4 ) == 0 ) ) )
+            if ( ( arealength == 5 ) && ( type == QLatin1String( "rect" ) || type == QLatin1String( "oval" ) ) )
             {
                 link->m_point = QPoint( miniexp_to_int( miniexp_nth( 1, area ) ), miniexp_to_int( miniexp_nth( 2, area ) ) );
                 link->m_size = QSize( miniexp_to_int( miniexp_nth( 3, area ) ), miniexp_to_int( miniexp_nth( 4, area ) ) );
-                if ( qstrncmp( miniexp_to_name( miniexp_nth( 0, area ) ), "rect", 4 ) == 0 )
+                if ( type == QLatin1String( "rect" ) )
                 {
                     link->m_area = KDjVu::Link::RectArea;
                 }
@@ -516,8 +671,7 @@ QList<KDjVu::Link*> KDjVu::linksForPage( int pageNum ) const
                 }
             }
             else if ( ( arealength > 0 ) && ( arealength % 2 == 1 ) &&
-                      miniexp_symbolp( miniexp_nth( 0, area ) ) &&
-                      ( qstrncmp( miniexp_to_name( miniexp_nth( 0, area ) ), "poly", 4 ) == 0 ) )
+                      type == QLatin1String( "poly" ) )
             {
                 link->m_area = KDjVu::Link::PolygonArea;
                 QPolygon poly;
@@ -527,13 +681,51 @@ QList<KDjVu::Link*> KDjVu::linksForPage( int pageNum ) const
                 }
                 link->m_poly = poly;
             }
-            // TODO: other link shapes
 
             if ( link->m_area != KDjVu::Link::UnknownArea )
-                ret.append( link );
+                links->append( link );
+        }
+        else if ( ann /* safety check */ && annotations )
+        {
+            if ( type == QLatin1String( "text" ) )
+            {
+                KDjVu::TextAnnotation* txtann = (KDjVu::TextAnnotation*)ann;
+                txtann->m_color.setRgb( 255, 255, 255, 0 );
+                for ( int j = 4; j < num; ++j )
+                {
+                    miniexp_t curelem = miniexp_nth( j, cur );
+                    if ( !miniexp_listp( curelem ) )
+                        continue;
+
+                    QString id = QString::fromUtf8( miniexp_to_name( miniexp_nth( 0, curelem ) ) );
+                    if ( id == QLatin1String( "backclr" ) )
+                        txtann->m_color.setNamedColor( QString::fromUtf8( miniexp_to_name( miniexp_nth( 1, curelem ) ) ) );
+                    else if ( id == QLatin1String( "pushpin" ) )
+                        txtann->m_inlineText = false;
+                }
+            }
+            else if ( type == QLatin1String( "line" ) )
+            {
+                KDjVu::LineAnnotation* lineann = (KDjVu::LineAnnotation*)ann;
+                lineann->m_color = Qt::black;
+                for ( int j = 4; j < num; ++j )
+                {
+                    miniexp_t curelem = miniexp_nth( j, cur );
+                    if ( !miniexp_listp( curelem ) )
+                        continue;
+
+                    QString id = QString::fromUtf8( miniexp_to_name( miniexp_nth( 0, curelem ) ) );
+                    if ( id == QLatin1String( "lineclr" ) )
+                        lineann->m_color.setNamedColor( QString::fromUtf8( miniexp_to_name( miniexp_nth( 1, curelem ) ) ) );
+                    else if ( id == QLatin1String( "arrow" ) )
+                        lineann->m_isArrow = true;
+                    else if ( id == QLatin1String( "width" ) )
+                        lineann->m_width = miniexp_to_int( miniexp_nth( 1, curelem ) );
+                }
+            }
+            annotations->append( ann );
         }
     }
-    return ret;
 }
 
 const QVector<KDjVu::Page*> &KDjVu::pages() const
@@ -671,5 +863,42 @@ void KDjVu::requestImage( int page, int width, int height, int rotation )
     emit imageGenerated( page, newimg );
 }
 
+bool KDjVu::exportAsPostScript( const QString & fileName, const QList<int>& pageList ) const
+{
+    if ( !d->m_djvu_document || fileName.trimmed().isEmpty() || pageList.isEmpty() )
+        return false;
+
+    QByteArray fn = QFile::encodeName( fileName );
+    FILE* f = fopen( fn.constData(), "w+" );
+    if ( !f )
+    {
+        kDebug() << "KDjVu::exportAsPostScript(): error while opening the file" << endl;
+        return false;
+    }
+
+    QString pl;
+    foreach ( int p, pageList )
+    {
+        if ( !pl.isEmpty() )
+            pl += QString::fromLatin1( "," );
+        pl += QString::number( p );
+    }
+    pl.prepend( "-page=" );
+
+    // setting the options
+    static const int optc = 1;
+    const char ** optv = (const char**)malloc( 1 * sizeof( char* ) );
+    QByteArray plb = pl.toAscii();
+    optv[0] = plb.constData();
+
+    ddjvu_job_t *printjob = ddjvu_document_print( d->m_djvu_document, f, optc, optv );
+    while ( !ddjvu_job_done( printjob ) )
+       handle_ddjvu_messages( d->m_djvu_cxt, true );
+
+    free( optv );
+
+    return fclose( f ) == 0;
+}
+
 
 #include "kdjvu.moc"
