@@ -10,9 +10,11 @@
 #ifndef _KDJVU_
 #define _KDJVU_
 
+#include <qcolor.h>
 #include <qimage.h>
 #include <qobject.h>
 #include <qpolygon.h>
+#include <qvariant.h>
 #include <qvector.h>
 
 class QDomDocument;
@@ -110,6 +112,67 @@ class KDjVu : public QObject
         };
 
 
+        /**
+         * The base implementation for a DjVu annotation.
+         */
+        class Annotation
+        {
+            friend class KDjVu;
+
+            public:
+                virtual ~Annotation();
+
+                enum AnnotationType { TextAnnotation, LineAnnotation };
+                virtual int type() const = 0;
+                QPoint point() const;
+                QString comment() const;
+                QColor color() const;
+
+            private:
+                QPoint m_point;
+                QString m_comment;
+                QColor m_color;
+        };
+
+        /**
+         * A DjVu text annotation.
+         */
+        class TextAnnotation : public Annotation
+        {
+            friend class KDjVu;
+
+            public:
+                virtual int type() const;
+                QSize size() const;
+                bool inlineText() const;
+
+            private:
+                TextAnnotation();
+                QSize m_size;
+                bool m_inlineText;
+        };
+
+        /**
+         * A DjVu line annotation.
+         */
+        class LineAnnotation : public Annotation
+        {
+            friend class KDjVu;
+
+            public:
+                virtual int type() const;
+                QPoint point2() const;
+                bool isArrow() const;
+                int width() const;
+
+            private:
+                LineAnnotation();
+                QPoint m_point2;
+                bool m_isArrow;
+                int m_width;
+        };
+
+
         /**
          * Opens the file \p fileName, closing the old one if necessary.
          */
@@ -127,12 +190,12 @@ class KDjVu : public QObject
         const QVector<KDjVu::Page*> &pages() const;
 
         /**
-         * Get the metadata for the specified \p key, or a null string otherwise.
+         * Get the metadata for the specified \p key, or a null variant otherwise.
          */
-        QString metaData( const QString & key ) const;
+        QVariant metaData( const QString & key ) const;
 
         /**
-         * Get ax XML document with the bookmarks of the current document (if any).
+         * Get an XML document with the bookmarks of the current document (if any).
          * The XML will look like this:
          * \verbatim
          * <!DOCTYPE KDjVuBookmarks>
@@ -146,9 +209,13 @@ class KDjVu : public QObject
         const QDomDocument * documentBookmarks() const;
 
         /**
-         * Return the links for the page \p pageNum
+         * Reads the links and the annotations for the page \p pageNum
+         *
+         * For both \p links and \p annotations , you can pass either a valid pointer
+         * (in case you want to extract that kind of information), or a null pointer
+         * (if you don't want that information).
          */
-        QList<KDjVu::Link*> linksForPage( int pageNum ) const;
+        void linksAndAnnotationsForPage( int pageNum, QList<KDjVu::Link*> *links, QList<KDjVu::Annotation*> *annotations ) const;
 
         // image handling
         /**
@@ -163,6 +230,12 @@ class KDjVu : public QObject
          */
         void requestImage( int page, int width, int height, int rotation );
 
+        /**
+         * Export the currently open document as PostScript file \p fileName.
+         * \returns whether the exporting was successful
+          */
+        bool exportAsPostScript( const QString & fileName, const QList<int>& pageList ) const;
+
     signals:
         /**
          * The image \p pix for page \p page was generated.
