@@ -7,10 +7,14 @@
  *   (at your option) any later version.                                   *
  ***************************************************************************/
 
+#include <okular/core/document.h>
+#include <okular/core/link.h>
+#include <okular/core/page.h>
+#include <okular/core/sourcereference.h>
+#include <okular/core/textpage.h>
+#include <okular/core/utils.h>
+
 #include "generator_dvi.h"
-#include "core/page.h"
-#include "core/link.h"
-#include "core/utils.h"
 #include "dviFile.h"
 #include "dviPageInfo.h"
 #include "dviRenderer.h"
@@ -31,6 +35,7 @@ OKULAR_EXPORT_PLUGIN(DviGenerator)
 DviGenerator::DviGenerator() : Okular::Generator(),
   m_docInfo( 0 ), m_docSynopsis( 0 ), ready( false ), m_dviRenderer( 0 )
 {
+    setFeature( TextExtraction );
 }
 
 bool DviGenerator::loadDocument( const QString & fileName, QVector< Okular::Page * > &pagesVector )
@@ -44,7 +49,7 @@ bool DviGenerator::loadDocument( const QString & fileName, QVector< Okular::Page
 
     kDebug() << "# of pages: " << m_dviRenderer->dviFile->total_pages << endl;
 
-    m_resolution = Okular::Utils::getDpiY();
+    m_resolution = Okular::Utils::dpiY();
     loadPages( pagesVector, 0 );
 
     ready = true;
@@ -65,7 +70,7 @@ bool DviGenerator::closeDocument()
     return true;
 }
 
-bool DviGenerator::canGeneratePixmap ( bool /*async*/ ) const
+bool DviGenerator::canGeneratePixmap () const
 {
     return ready;
 }
@@ -220,7 +225,7 @@ void DviGenerator::generatePixmap( Okular::PixmapRequest *request )
         {
             kDebug() << "Image OK" << endl;
 
-            request->page()->setImage( request->id(), pageInfo->img );
+            request->page()->setPixmap( request->id(), new QPixmap( QPixmap::fromImage( pageInfo->img ) ) );
 
             request->page()->setObjectRects(
                      generateDviLinks( pageInfo, 0 ) );
@@ -231,17 +236,12 @@ void DviGenerator::generatePixmap( Okular::PixmapRequest *request )
 
     delete pageInfo;
 
-    signalRequestDone( request );
+    signalPixmapRequestDone( request );
 }
 
-bool DviGenerator::canGenerateTextPage() const
+Okular::TextPage* DviGenerator::textPage( Okular::Page *page )
 {
-    return true;
-}
-
-void DviGenerator::generateSyncTextPage( Okular::Page* page )
-{
-    kDebug() << "DviGenerator::generateSyncTextPage( Okular::Page * page )" << endl;
+    kDebug() << "DviGenerator::textPage( Okular::Page * page )" << endl;
     dviPageInfo *pageInfo = new dviPageInfo();
     pageSize ps;
 
@@ -255,14 +255,15 @@ void DviGenerator::generateSyncTextPage( Okular::Page* page )
     pageInfo->resolution = (double)(pageInfo->width)/ps.width().getLength_in_inch();
 
     // get page text from m_dviRenderer
+    Okular::TextPage *ktp = 0;
     if ( m_dviRenderer )
     {
         m_dviRenderer->getText( pageInfo );
 
-        Okular::TextPage *ktp = extractTextFromPage( pageInfo, page->totalOrientation() );
-        page->setSearchPage( ktp );
+        ktp = extractTextFromPage( pageInfo, page->totalOrientation() );
     }
     delete pageInfo;
+    return ktp;
 }
 
 Okular::TextPage *DviGenerator::extractTextFromPage( dviPageInfo *pageInfo, int orientation )
@@ -411,10 +412,26 @@ void DviGenerator::loadPages( QVector< Okular::Page * > &pagesVector, int orient
         Okular::Page * page = new Okular::Page( i,
                                         pageRequiredSize.width(),
                                         pageRequiredSize.height(),
-                                        orientation );
+                                        (Okular::Rotation)orientation );
         pagesVector[i] = page;
     }
     kDebug() << "pagesVector successfully inizialized ! " << endl;
+
+    // filling the pages with the source references rects
+    const QVector<DVI_SourceFileAnchor>& sourceAnchors = m_dviRenderer->sourceAnchors();
+    QVector< QLinkedList< Okular::SourceRefObjectRect * > > refRects( numofpages );
+    foreach ( const DVI_SourceFileAnchor& sfa, sourceAnchors )
+    {
+        if ( sfa.page < 1 || (int)sfa.page > numofpages )
+            continue;
+
+        Okular::NormalizedPoint p( 0.5, (double)sfa.distance_from_top.getLength_in_pixel( Okular::Utils::dpiY() ) / (double)pageRequiredSize.height() );
+        Okular::SourceReference * sourceRef = new Okular::SourceReference( sfa.fileName, sfa.line );
+        refRects[ sfa.page - 1 ].append( new Okular::SourceRefObjectRect( p, sourceRef ) );
+    }
+    for ( int i = 0; i < refRects.size(); ++i )
+        if ( !refRects.at(i).isEmpty() )
+            pagesVector[i]->setSourceReferences( refRects.at(i) );
 }
 
 #include "generator_dvi.moc"
