@@ -10,7 +10,9 @@
 #ifndef _OKULAR_GENERATOR_GHOSTVIEW_H_
 #define _OKULAR_GENERATOR_GHOSTVIEW_H_
 
-#include "core/generator.h"
+#include <okular/core/generator.h>
+#include <okular/interfaces/guiinterface.h>
+#include <okular/interfaces/configinterface.h>
 
 class GSInterpreterCMD;
 class GSLogWindow;
@@ -19,9 +21,12 @@ class GSInternalDocument;
 class KTempFile;
 class KActionCollection;
 
-class GSGenerator : public Okular::Generator
+class GSGenerator : public Okular::Generator, public Okular::ConfigInterface, public Okular::GuiInterface
 {
     Q_OBJECT
+    Q_INTERFACES( Okular::ConfigInterface )
+    Q_INTERFACES( Okular::GuiInterface )
+
     public:
         /** virtual methods to reimplement **/
         // load a document and fill up the pagesVector
@@ -34,24 +39,24 @@ class GSGenerator : public Okular::Generator
         const Okular::DocumentFonts * generateDocumentFonts() { return 0L; }
 
         // page contents generation
-        bool canGeneratePixmap( bool async ) const;
-        void generatePixmap( Okular::PixmapRequest * request ) ;
+        bool canGeneratePixmap() const;
+        void generatePixmap( Okular::PixmapRequest * request );
 
-        bool supportsRotation() const { return true; } ;
+        // page size management
+        Okular::PageSize::List pageSizes() const;
+        void pageSizeChanged( const Okular::PageSize &, const Okular::PageSize & );
 
-        // paper size management
-        bool supportsPaperSizes() const;
-        QStringList paperSizes() const;
-        void setPaperSize( QVector<Okular::Page*> & pagesVector, int newsize );
-
-        void setupGUI(KActionCollection  * /*ac*/ , QToolBox * /* tBox */);
-        void freeGUI();
+        QString xmlFile() const;
+        void setupGui( KActionCollection *, QToolBox * );
+        void freeGui();
 
         // print document using already configured kprinter
         bool print( KPrinter& /*printer*/ );
         QString fileName() const;
 
+        bool reparseConfig();
         void addPages( KConfigDialog* dlg );
+
         /** constructor **/
         GSGenerator();
         ~GSGenerator();
@@ -91,6 +96,8 @@ class GSGenerator : public Okular::Generator
         GSLogWindow * m_logWindow ;
         KActionCollection* m_actionCollection;
         QToolBox * m_box;
+
+        Okular::PageSize::List m_pageSizes;
 };
 
 #endif
