@@ -10,7 +10,9 @@
 #ifndef _OKULAR_GENERATOR_KIMGIO_H_
 #define _OKULAR_GENERATOR_KIMGIO_H_
 
-#include "core/generator.h"
+#include <core/generator.h>
+
+#include <QtGui/QImage>
 
 class KIMGIOGenerator : public Okular::Generator
 {
@@ -21,20 +23,17 @@ class KIMGIOGenerator : public Okular::Generator
 
         // [INHERITED] load a document and fill up the pagesVector
         bool loadDocument( const QString & fileName, QVector<Okular::Page*> & pagesVector );
+        bool loadDocumentFromData( const QByteArray & fileData, QVector<Okular::Page*> & pagesVector );
         bool closeDocument();
 
-        // [INHERITED] perform actions on document / pages
-        bool canGeneratePixmap( bool async ) const;
-        void generatePixmap( Okular::PixmapRequest * request );
-
-        // [INHERITED] rotation capability
-        bool supportsRotation() const { return true; };
-
         // [INHERITED] print document using already configured kprinter
         bool print( KPrinter& printer );
 
+    protected:
+        QImage image( Okular::PixmapRequest * request );
+
     private:
-        QPixmap *m_pix;
+        QImage m_img;
 };
 
 #endif
