@@ -9,28 +9,33 @@
  ***************************************************************************/
 
 // qt/kde includes
+#include <qcolor.h>
 #include <qfile.h>
 #include <qimage.h>
 #include <qregexp.h>
 #include <qtextstream.h>
-#include <kauthorized.h>
 #include <klocale.h>
+#include <kmessagebox.h>
 #include <kpassworddialog.h>
 #include <kwallet.h>
 #include <kprinter.h>
 #include <ktemporaryfile.h>
 #include <kdebug.h>
+#include <kglobal.h>
+
+#include <okular/core/document.h>
+#include <okular/core/link.h>
+#include <okular/core/page.h>
+#include <okular/core/annotations.h>
+#include <okular/core/pagetransition.h>
+#include <okular/core/sound.h>
+#include <okular/core/sourcereference.h>
+#include <okular/core/textpage.h>
 
 // local includes
 #include "generator_pdf.h"
-#include "core/observer.h" //for PAGEVIEW_ID
-#include "core/page.h"
-#include "core/annotations.h"
-#include "core/pagetransition.h"
-#include "core/sound.h"
 #include "settings.h"
 
-#include <config.h>
 #include <config-okular.h>
 
 class PDFEmbeddedFile : public Okular::EmbeddedFile
@@ -73,7 +78,7 @@ static void fillViewportFromLinkDestination( Okular::DocumentViewport &viewport,
 {
     viewport.pageNumber = destination.pageNumber() - 1;
 
-    if (viewport.pageNumber == -1) return;
+    if (!viewport.isValid()) return;
 
     // get destination position
     // TODO add other attributes to the viewport (taken from link)
@@ -222,6 +227,10 @@ PDFGenerator::PDFGenerator()
     pixmapRequest( 0 ), docInfoDirty( true ), docSynopsisDirty( true ),
     docFontsDirty( true ), docEmbeddedFilesDirty( true )
 {
+    setFeature( TextExtraction );
+#ifdef HAVE_POPPLER_HEAD
+    setFeature( ReadRawData );
+#endif
     // update the configuration
     reparseConfig();
     // generate the pixmapGeneratorThread
@@ -251,7 +260,36 @@ bool PDFGenerator::loadDocument( const QString & filePath, QVector<Okular::Page*
 #endif
     // create PDFDoc for the given file
     pdfdoc = Poppler::Document::load( filePath, 0, 0 );
+    bool success = init(pagesVector, filePath.section('/', -1, -1));
+    if (success && QFile::exists(filePath + QLatin1String( "sync" )))
+    {
+        loadPdfSync(filePath, pagesVector);
+    }
+    return success;
+}
 
+bool PDFGenerator::loadDocumentFromData( const QByteArray & fileData, QVector<Okular::Page*> & pagesVector )
+{
+#ifdef HAVE_POPPLER_HEAD
+#ifndef NDEBUG
+    if ( pdfdoc )
+    {
+        kDebug() << "PDFGenerator: multiple calls to loadDocument. Check it." << endl;
+        return false;
+    }
+#endif
+    // create PDFDoc for the given file
+    pdfdoc = Poppler::Document::loadFromData( fileData, 0, 0 );
+    return init(pagesVector, QString());
+#else
+    Q_UNUSED(fileData)
+    Q_UNUSED(pagesVector)
+    return false;
+#endif
+}
+
+bool PDFGenerator::init(QVector<Okular::Page*> & pagesVector, const QString &walletKey)
+{
     // if the file didn't open correctly it might be encrypted, so ask for a pass
     bool firstInput = true;
     bool triedWallet = false;
@@ -259,10 +297,10 @@ bool PDFGenerator::loadDocument( const QString & filePath, QVector<Okular::Page*
     bool keep = true;
     while ( pdfdoc && pdfdoc->isLocked() )
     {
-        QByteArray password;
+        QString password;
 
         // 1.A. try to retrieve the first password from the kde wallet system
-        if ( !triedWallet )
+        if ( !triedWallet && !walletKey.isNull() )
         {
             QString walletName = KWallet::Wallet::NetworkWallet();
             wallet = KWallet::Wallet::openWallet( walletName );
@@ -275,8 +313,8 @@ bool PDFGenerator::loadDocument( const QString & filePath, QVector<Okular::Page*
 
                 // look for the pass in that folder
                 QString retrievedPass;
-                if ( !wallet->readPassword( filePath.section('/', -1, -1), retrievedPass ) )
-                    password = retrievedPass.toLocal8Bit();
+                if ( !wallet->readPassword( walletKey, retrievedPass ) )
+                    password = retrievedPass;
             }
             triedWallet = true;
         }
@@ -292,18 +330,23 @@ bool PDFGenerator::loadDocument( const QString & filePath, QVector<Okular::Page*
             firstInput = false;
 
             // if the user presses cancel, abort opening
-            if ( KPasswordDialog::getPassword( 0, password, i18n( "Document Password" ), prompt, wallet ? &keep : 0 ) != KPasswordDialog::Accepted )
+            KPasswordDialog dlg( 0, wallet ? KPasswordDialog::ShowKeepPassword : KPasswordDialog::KPasswordDialogFlags() );
+            dlg.setCaption( i18n( "Document Password" ) );
+            dlg.setPrompt( prompt );
+            if( !dlg.exec() )
                 break;
+            password = dlg.password();
+            if ( wallet )
+                keep = dlg.keepPassword();
         }
 
         // 2. reopen the document using the password
-        pdfdoc->unlock( password, password );
+        pdfdoc->unlock( password.toLocal8Bit(), password.toLocal8Bit() );
 
         // 3. if the password is correct and the user chose to remember it, store it to the wallet
         if ( !pdfdoc->isLocked() && wallet && /*safety check*/ wallet->isOpen() && keep )
         {
-            QString goodPass = QString::fromLocal8Bit( password.data() );
-            wallet->writePassword( filePath.section('/', -1, -1), goodPass );
+            wallet->writePassword( walletKey, password );
         }
     }
     if ( !pdfdoc || pdfdoc->isLocked() )
@@ -316,6 +359,7 @@ bool PDFGenerator::loadDocument( const QString & filePath, QVector<Okular::Page*
     // build Pages (currentPage was set -1 by deletePages)
     uint pageCount = pdfdoc->numPages();
     pagesVector.resize(pageCount);
+    rectsGenerated.fill(false, pageCount);
 
     loadPages(pagesVector, 0, false);
 
@@ -351,13 +395,13 @@ void PDFGenerator::loadPages(QVector<Okular::Page*> &pagesVector, int rotation,
         QSize pSize = p->pageSize();
         w = pSize.width();
         h = pSize.height();
-        int orientation = 0;
+        Okular::Rotation orientation = Okular::Rotation0;
         switch (p->orientation())
         {
-          case Poppler::Page::Landscape: orientation = 1; break;
-          case Poppler::Page::UpsideDown: orientation = 2; break;
-          case Poppler::Page::Seascape: orientation = 3; break;
-          case Poppler::Page::Portrait: orientation = 0; break;
+          case Poppler::Page::Landscape: orientation = Okular::Rotation90; break;
+          case Poppler::Page::UpsideDown: orientation = Okular::Rotation180; break;
+          case Poppler::Page::Seascape: orientation = Okular::Rotation270; break;
+          case Poppler::Page::Portrait: orientation = Okular::Rotation0; break;
         }
         if (rotation % 2 == 1)
           qSwap(w,h);
@@ -369,14 +413,24 @@ void PDFGenerator::loadPages(QVector<Okular::Page*> &pagesVector, int rotation,
 #ifdef HAVE_POPPLER_HEAD
         Poppler::Link * tmplink = p->action( Poppler::Page::Opening );
         if ( tmplink )
+        {
             page->setPageAction( Okular::Page::Opening, createLinkFromPopplerLink( tmplink, pdfdoc ) );
+            delete tmplink;
+        }
         tmplink = p->action( Poppler::Page::Closing );
         if ( tmplink )
+        {
             page->setPageAction( Okular::Page::Closing, createLinkFromPopplerLink( tmplink, pdfdoc ) );
+            delete tmplink;
+        }
+        page->setDuration( p->duration() );
+        page->setLabel( p->label() );
 #endif
 // 	    kWarning() << page->width() << "x" << page->height() << endl;
 
+#ifdef PDFGENERATOR_DEBUG
 	kDebug() << "load page " << i << " with rotation " << rotation << " and orientation " << orientation << endl;
+#endif
 	delete p;
 
         if (clear && pagesVector[i])
@@ -531,21 +585,17 @@ const QList<Okular::EmbeddedFile*> *PDFGenerator::embeddedFiles() const
     return &docEmbeddedFiles;
 }
 
-bool PDFGenerator::isAllowed( int permissions ) const
+bool PDFGenerator::isAllowed( Okular::Permissions permissions ) const
 {
-#if !OKULAR_FORCE_DRM
-    if (KAuthorized::authorize("skip_drm") && !Okular::Settings::obeyDRM()) return true;
-#endif
-
     bool b = true;
-    if (permissions & Okular::Document::AllowModify) b = b && pdfdoc->okToChange();
-    if (permissions & Okular::Document::AllowCopy) b = b && pdfdoc->okToCopy();
-    if (permissions & Okular::Document::AllowPrint) b = b && pdfdoc->okToPrint();
-    if (permissions & Okular::Document::AllowNotes) b = b && pdfdoc->okToAddNotes();
+    if (permissions & Okular::AllowModify) b = b && pdfdoc->okToChange();
+    if (permissions & Okular::AllowCopy) b = b && pdfdoc->okToCopy();
+    if (permissions & Okular::AllowPrint) b = b && pdfdoc->okToPrint();
+    if (permissions & Okular::AllowNotes) b = b && pdfdoc->okToAddNotes();
     return b;
 }
 
-bool PDFGenerator::canGeneratePixmap( bool /* async */) const
+bool PDFGenerator::canGeneratePixmap() const
 {
     return ready;
 }
@@ -574,15 +624,22 @@ void PDFGenerator::generatePixmap( Okular::PixmapRequest * request )
     /** synchronous request: in-place generation **/
     // compute dpi used to get an image with desired width and height
     Okular::Page * page = request->page();
-    double fakeDpiX = request->width() * 72.0 / page->width(),
-           fakeDpiY = request->height() * 72.0 / page->height();
+
+    double pageWidth = page->width(),
+           pageHeight = page->height();
+
+    if ( page->rotation() % 2 )
+        qSwap( pageWidth, pageHeight );
+
+    double fakeDpiX = request->width() * 72.0 / pageWidth,
+           fakeDpiY = request->height() * 72.0 / pageHeight;
 
     // setup Okular:: output device: text page is generated only if we are at 72dpi.
     // since we can pre-generate the TextPage at the right res.. why not?
-    bool genTextPage = !page->hasSearchPage() && (request->width() == page->width()) &&
+    bool genTextPage = !page->hasTextPage() && (request->width() == page->width()) &&
                        (request->height() == page->height());
-    // generate links and image rects if rendering pages on pageview
-    bool genObjectRects = request->id() & (PAGEVIEW_ID | PRESENTATION_ID);
+    // generate links rects only the first time
+    bool genObjectRects = !rectsGenerated.at( page->number() );
 
     // 0. LOCK [waits for the thread end]
     docLock.lock();
@@ -592,7 +649,11 @@ void PDFGenerator::generatePixmap( Okular::PixmapRequest * request )
     Poppler::Page *p = pdfdoc->page(page->number());
 
     // 2. Take data from outputdev and attach it to the Page
-    page->setImage( request->id(), p->splashRenderToImage(fakeDpiX, fakeDpiY, -1, -1, -1, -1, genObjectRects, (Poppler::Page::Rotation)document()->rotation()) );
+#ifdef HAVE_POPPLER_HEAD
+    page->setPixmap( request->id(), new QPixmap( QPixmap::fromImage( p->renderToImage(fakeDpiX, fakeDpiY, -1, -1, request->width(), request->height(), genObjectRects, Poppler::Page::Rotate0 ) ) ) );
+#else
+    page->setPixmap( request->id(), p->splashRenderToPixmap(fakeDpiX, fakeDpiY, -1, -1, request->width(), request->height(), genObjectRects, Poppler::Page::Rotate0 ) );
+#endif
     
     if ( genObjectRects )
     {
@@ -600,14 +661,15 @@ void PDFGenerator::generatePixmap( Okular::PixmapRequest * request )
         // and as we are not doing anything with Image type rects i did not port it, have a look at
         // dead gp_outputdev.cpp on image extraction
         page->setObjectRects( generateLinks(p->links(), request->width(), request->height(), pdfdoc) );
+        rectsGenerated[ request->page()->number() ] = true;
     }
 
     // 3. UNLOCK [re-enables shared access]
     docLock.unlock();
     if ( genTextPage )
     {
-        QList<Poppler::TextBox*> textList = p->textList((Poppler::Page::Rotation)request->page()->rotation());
-        page->setSearchPage( abstractTextPage(textList, page->height(), page->width(), request->page()->totalOrientation()) );
+        QList<Poppler::TextBox*> textList = p->textList((Poppler::Page::Rotation)request->page()->orientation());
+        page->setTextPage( abstractTextPage(textList, page->height(), page->width(), request->page()->orientation()) );
         qDeleteAll(textList);
     }
     delete p;
@@ -616,26 +678,25 @@ void PDFGenerator::generatePixmap( Okular::PixmapRequest * request )
     ready = true;
 
     // notify the new generation
-    signalRequestDone( request );
+    signalPixmapRequestDone( request );
 }
 
-bool PDFGenerator::canGenerateTextPage() const
+Okular::TextPage* PDFGenerator::textPage( Okular::Page *page )
 {
-    return true;
-}
-
-void PDFGenerator::generateSyncTextPage( Okular::Page * page )
-{
-    kDebug() << "calling generateSyncTextPage( Okular::Page * page )" << endl;
+    kDebug() << "calling textPage( Okular::Page * page )" << endl;
     // build a TextList...
     Poppler::Page *pp = pdfdoc->page( page->number() );
     docLock.lock();
-    QList<Poppler::TextBox*> textList = pp->textList((Poppler::Page::Rotation)page->rotation());
+    QList<Poppler::TextBox*> textList = pp->textList((Poppler::Page::Rotation)page->orientation());
     docLock.unlock();
     delete pp;
-    // ..and attach it to the page
-    page->setSearchPage( abstractTextPage(textList, page->height(), page->width(), page->totalOrientation()) );
+
+    const double pageWidth = ( page->rotation() % 2 ? page->height() : page->width() );
+    const double pageHeight = ( page->rotation() % 2 ? page->width() : page->height() );
+
+    Okular::TextPage *tp = abstractTextPage(textList, pageHeight, pageWidth, (Poppler::Page::Rotation)page->orientation());
     qDeleteAll(textList);
+    return tp;
 }
 
 bool PDFGenerator::print( KPrinter& printer )
@@ -643,6 +704,11 @@ bool PDFGenerator::print( KPrinter& printer )
     int width, height;
     QString ps = printer.option( "PageSize" );
     QRegExp sizere( "w(\\d+)h(\\d+)" );
+    int marginTop, marginLeft, marginRight, marginBottom;
+    marginTop = (int)printer.option("kde-margin-top").toDouble();
+    marginLeft = (int)printer.option("kde-margin-left").toDouble();
+    marginRight = (int)printer.option("kde-margin-right").toDouble();
+    marginBottom = (int)printer.option("kde-margin-bottom").toDouble();
     if ( sizere.exactMatch( ps ) )
     {
         // size not supported by Qt, KPrinter gives us the size as wWIDTHhHEIGHT
@@ -674,12 +740,31 @@ bool PDFGenerator::print( KPrinter& printer )
     
     docLock.lock();
     // TODO rotation
+#ifdef HAVE_POPPLER_HEAD
+    double xScale = ((double)width - (double)marginLeft - (double)marginRight) / (double)width;
+    double yScale = ((double)height - (double)marginBottom - (double)marginTop) / (double)height;
+    bool strictMargins = false;
+    if ( abs((int)(xScale * 100) - (int)(yScale * 100)) > 5 ) {
+        int result = KMessageBox::questionYesNo(0,
+                                               i18n("The margins you specified are changing the page aspect ratio. Do you want to print with the aspect ratio changed or do you want the margins to be adapted so that aspect ratio is preserved?"),
+                                               i18n("Aspect ratio change"),
+                                               KGuiItem( i18n("Print with specified margins") ),
+                                               KGuiItem( i18n("Print adapting margins to keep aspect ratio") ),
+                                               "kpdfStrictlyObeyMargins");
+        if (result == KMessageBox::Yes) strictMargins = true;
+    }
+    if (pdfdoc->print(tempfilename, pageList, 72, 72, 0, width, height, marginRight, marginBottom, marginLeft, marginTop, strictMargins))
+    {
+        docLock.unlock();
+        return printer.printFiles(QStringList(tempfilename), true);
+    }
+#else
     if (pdfdoc->print(tempfilename, pageList, 72, 72, 0, width, height))
     {
         docLock.unlock();
-        printer.printFiles(QStringList(tempfilename), true);
-        return true;
+        return printer.printFiles(QStringList(tempfilename), true);
     }
+#endif
     else
     {
         docLock.unlock();
@@ -745,6 +830,10 @@ bool PDFGenerator::reparseConfig()
     return false;
 }
 
+void PDFGenerator::addPages( KConfigDialog * )
+{
+}
+
 Okular::ExportFormat::List PDFGenerator::exportFormats() const
 {
     static Okular::ExportFormat::List formats;
@@ -992,7 +1081,9 @@ void PDFGenerator::addAnnotations( Poppler::Page * popplerPage, Okular::Page * p
         if (newann)
         {
             // the Contents field has lines separated by \r
-            newann->contents.replace( QLatin1Char( '\r' ), QLatin1Char( '\n' ) );
+            QString contents = newann->contents();
+            contents.replace( QLatin1Char( '\r' ), QLatin1Char( '\n' ) );
+            newann->setContents( contents );
             page->addAnnotation(newann);
         }
     }
@@ -1073,6 +1164,137 @@ void PDFGenerator::addTransition( Poppler::Page * pdfPage, Okular::Page * page )
     page->setTransition( transition );
 }
 
+struct pdfsyncpoint
+{
+    QString file;
+    qlonglong x;
+    qlonglong y;
+    int row;
+    int column;
+    int page;
+};
+
+void PDFGenerator::loadPdfSync( const QString & filePath, QVector<Okular::Page*> & pagesVector )
+{
+    QFile f( filePath + QLatin1String( "sync" ) );
+    if ( !f.open( QIODevice::ReadOnly ) )
+        return;
+
+    QTextStream ts( &f );
+    // first row: core name of the pdf output - we skip it
+    ts.readLine();
+    // second row: version string, in the form 'Version %u'
+    QString versionstr = ts.readLine();
+    QRegExp versionre( "Version (\\d+)" );
+    versionre.setCaseSensitivity( Qt::CaseInsensitive );
+    if ( !versionre.exactMatch( versionstr ) )
+        return;
+
+    QHash<int, pdfsyncpoint> points;
+    QString currentfile;
+    int currentpage = -1;
+    QRegExp newfilere( "\\(\\s*([^\\s]+)" );
+    QRegExp linere( "l\\s+(\\d+)\\s+(\\d+)(\\s+(\\d+))?" );
+    QRegExp pagere( "s\\s+(\\d+)" );
+    QRegExp locre( "p\\s+(\\d+)\\s+(\\d+)\\s+(\\d+)" );
+    QRegExp locstarre( "p\\*\\s+(\\d+)\\s+(\\d+)\\s+(\\d+)" );
+
+    QString line;
+    while ( !ts.atEnd() )
+    {
+        line = ts.readLine();
+        if ( line.startsWith( QLatin1Char( 'l' ) ) && linere.exactMatch( line ) )
+        {
+            int id = linere.cap( 1 ).toInt();
+            QHash<int, pdfsyncpoint>::const_iterator it = points.find( id );
+            if ( it == points.end() )
+            {
+                pdfsyncpoint pt;
+                pt.x = 0;
+                pt.y = 0;
+                pt.row = linere.cap( 2 ).toInt();
+                pt.column = 0; // TODO
+                pt.page = -1;
+                pt.file = currentfile;
+                points[ id ] = pt;
+            }
+        }
+        else if ( line.startsWith( QLatin1Char( 's' ) ) && pagere.exactMatch( line ) )
+        {
+            currentpage = pagere.cap( 1 ).toInt() - 1;
+        }
+        else if ( line.startsWith( QLatin1String( "p*" ) ) && locstarre.exactMatch( line ) )
+        {
+            // TODO
+            kDebug(4651) << "PdfSync: 'p*' line ignored" << endl;
+        }
+        else if ( line.startsWith( QLatin1Char( 'p' ) ) && locre.exactMatch( line ) )
+        {
+            int id = locre.cap( 1 ).toInt();
+            QHash<int, pdfsyncpoint>::iterator it = points.find( id );
+            if ( it != points.end() )
+            {
+                it->x = locre.cap( 2 ).toInt();
+                it->y = locre.cap( 3 ).toInt();
+                it->page = currentpage;
+            }
+        }
+        else if ( line.startsWith( QLatin1Char( '(' ) ) && newfilere.exactMatch( line ) )
+        {
+            QString newfile = newfilere.cap( 1 );
+            if ( currentfile.isEmpty() )
+            {
+                currentfile = newfile;
+            }
+            else
+                kDebug(4651) << "PdfSync: more than one file level: " << newfile << endl;
+        }
+        else if ( line == QLatin1String( ")" ) )
+        {
+            if ( !currentfile.isEmpty() )
+            {
+                currentfile.clear();
+            }
+            else
+                kDebug(4651) << "PdfSync: going one level down: " << currentfile << endl;
+        }
+        else
+            kDebug(4651) << "PdfSync: unknown line format: '" << line << "'" << endl;
+
+    }
+
+    QVector< QLinkedList< Okular::SourceRefObjectRect * > > refRects( pagesVector.size() );
+    foreach ( const pdfsyncpoint& pt, points )
+    {
+        // drop pdfsync points not completely valid
+        if ( pt.page < 0 || pt.page >= pagesVector.size() )
+            continue;
+
+        // maginc numbers for TeX's RSU's (Ridiculously Small Units) conversion to pixels
+        Okular::NormalizedPoint p(
+            ( pt.x * 72.0 ) / ( 72.27 * 65536.0 * pagesVector[pt.page]->width() ),
+            ( pt.y * 72.0 ) / ( 72.27 * 65536.0 * pagesVector[pt.page]->height() )
+            );
+        QString file;
+        if ( !pt.file.isEmpty() )
+        {
+            file = pt.file;
+            int dotpos = file.lastIndexOf( QLatin1Char( '.' ) );
+            QString ext;
+            if ( dotpos == -1 )
+                file += QString::fromLatin1( ".tex" );
+        }
+        else
+        {
+            file = filePath;
+        }
+        Okular::SourceReference * sourceRef = new Okular::SourceReference( file, pt.row, pt.column );
+        refRects[ pt.page ].append( new Okular::SourceRefObjectRect( p, sourceRef ) );
+    }
+    for ( int i = 0; i < refRects.size(); ++i )
+        if ( !refRects.at(i).isEmpty() )
+            pagesVector[i]->setSourceReferences( refRects.at(i) );
+}
 
 
 void PDFGenerator::threadFinished()
@@ -1112,16 +1334,22 @@ void PDFGenerator::threadFinished()
     QList<Poppler::TextBox*> outText = generatorThread->takeText();
     QLinkedList< Okular::ObjectRect * > outRects = generatorThread->takeObjectRects();
 
-    request->page()->setImage( request->id(), *outImage );
+    request->page()->setPixmap( request->id(), new QPixmap( QPixmap::fromImage( *outImage ) ) );
     delete outImage;
     if ( !outText.isEmpty() )
     {
-        request->page()->setSearchPage( abstractTextPage( outText , 
-            request->page()->height(), request->page()->width(),request->page()->totalOrientation()));
+        request->page()->setTextPage( abstractTextPage( outText , 
+            request->page()->height(), request->page()->width(),request->page()->orientation()));
         qDeleteAll(outText);
     }
-    bool genObjectRects = request->id() & (PAGEVIEW_ID | PRESENTATION_ID);
-    if (genObjectRects) request->page()->setObjectRects( outRects );
+    bool genObjectRects = !rectsGenerated.at( request->page()->number() );
+    if (genObjectRects)
+    {
+        request->page()->setObjectRects( outRects );
+        rectsGenerated[ request->page()->number() ] = true;
+    }
+    else
+        qDeleteAll( outRects );
 
     // 3. tell generator that data has been taken
     generatorThread->endGeneration();
@@ -1129,7 +1357,7 @@ void PDFGenerator::threadFinished()
     // update ready state
     ready = true;
     // notify the new generation
-    signalRequestDone( request );
+    signalPixmapRequestDone( request );
 }
 
 
@@ -1241,7 +1469,9 @@ QList<Poppler::TextBox*> PDFPixmapGeneratorThread::takeText()
 QLinkedList< Okular::ObjectRect * > PDFPixmapGeneratorThread::takeObjectRects() const
 {
     d->m_rectsTaken = true;
-    return d->m_rects;
+    QLinkedList< Okular::ObjectRect * > newrects = d->m_rects;
+    d->m_rects.clear();
+    return newrects;
 }
 
 void PDFPixmapGeneratorThread::run()
@@ -1252,17 +1482,23 @@ void PDFPixmapGeneratorThread::run()
     Okular::Page * page = d->currentRequest->page();
     int width = d->currentRequest->width(),
         height = d->currentRequest->height();
-    double fakeDpiX = width * 72.0 / page->width(),
-           fakeDpiY = height * 72.0 / page->height();
+    double pageWidth = page->width(),
+           pageHeight = page->height();
+
+    if ( page->rotation() % 2 )
+        qSwap( pageWidth, pageHeight );
+
+    double fakeDpiX = width * 72.0 / pageWidth,
+           fakeDpiY = height * 72.0 / pageHeight;
 
     // setup Okular:: output device: text page is generated only if we are at 72dpi.
     // since we can pre-generate the TextPage at the right res.. why not?
-    bool genTextPage = !page->hasSearchPage() &&
+    bool genTextPage = !page->hasTextPage() &&
                        ( width == page->width() ) &&
                        ( height == page->height() );
 
-    // generate links and image rects if rendering pages on pageview
-    bool genObjectRects = d->currentRequest->id() & (PAGEVIEW_ID | PRESENTATION_ID);
+    // generate links rects only the first time
+    bool genObjectRects = !d->generator->rectsGenerated.at( page->number() );
 
     // 0. LOCK s[tart locking XPDF thread unsafe classes]
     d->generator->docLock.lock();
@@ -1277,7 +1513,11 @@ void PDFPixmapGeneratorThread::run()
     if ( !d->m_textList.isEmpty() )
         kDebug() << "PDFPixmapGeneratorThread: previous text not taken" << endl;
 #endif
-    d->m_image = new QImage( pp->splashRenderToImage( fakeDpiX, fakeDpiY, -1, -1, -1, -1, genObjectRects, (Poppler::Page::Rotation)0 ) );
+#ifdef HAVE_POPPLER_HEAD
+    d->m_image = new QImage( pp->renderToImage( fakeDpiX, fakeDpiY, -1, -1, width, height, genObjectRects, Poppler::Page::Rotate0 ) );
+#else
+    d->m_image = new QImage( pp->splashRenderToImage( fakeDpiX, fakeDpiY, -1, -1, width, height, genObjectRects, Poppler::Page::Rotate0 ) );
+#endif
     
     if ( genObjectRects )
     {
@@ -1287,7 +1527,7 @@ void PDFPixmapGeneratorThread::run()
 
     if ( genTextPage )
     {
-        d->m_textList = pp->textList((Poppler::Page::Rotation)d->currentRequest->page()->rotation());
+        d->m_textList = pp->textList((Poppler::Page::Rotation)d->currentRequest->page()->orientation());
     }
     delete pp;
     
