@@ -16,13 +16,10 @@
 #include <poppler-qt4.h>
 
 #include <qmutex.h>
-#include <qcolor.h>
-#include <qstring.h>
 #include <qthread.h>
-#include "core/generator.h"
-#include "core/document.h"
-#include "core/link.h"
-#include "core/textpage.h"
+
+#include <okular/core/generator.h>
+#include <okular/interfaces/configinterface.h>
 
 namespace Okular {
 class ObjectRect;
@@ -44,15 +41,18 @@ class PDFPixmapGeneratorThread;
  * Background asynchronous contents providing is done via a QThread inherited
  * class defined at the bottom of the file.
  */
-class PDFGenerator : public Okular::Generator
+class PDFGenerator : public Okular::Generator, public Okular::ConfigInterface
 {
     Q_OBJECT
+    Q_INTERFACES( Okular::ConfigInterface )
+
     public:
         PDFGenerator();
         virtual ~PDFGenerator();
 
         // [INHERITED] load a document and fill up the pagesVector
         bool loadDocument( const QString & fileName, QVector<Okular::Page*> & pagesVector );
+        bool loadDocumentFromData( const QByteArray & fileData, QVector<Okular::Page*> & pagesVector );
         bool closeDocument();
         void loadPages(QVector<Okular::Page*> &pagesVector, int rotation=-1, bool clear=false);
         // [INHERITED] document information
@@ -63,16 +63,11 @@ class PDFGenerator : public Okular::Generator
         PageSizeMetric pagesSizeMetric() const { return Points; }
 
         // [INHERITED] document information
-        bool isAllowed( int permissions ) const;
+        bool isAllowed( Okular::Permissions permissions ) const;
 
         // [INHERITED] perform actions on document / pages
-        bool canGeneratePixmap( bool async ) const;
+        bool canGeneratePixmap() const;
         void generatePixmap( Okular::PixmapRequest * request );
-        bool canGenerateTextPage() const;
-        void generateSyncTextPage( Okular::Page * page );
-
-        bool supportsSearching() const { return true; };
-        bool supportsRotation() const { return true; };
 
         // [INHERITED] print page using an already configured kprinter
         bool print( KPrinter& printer );
@@ -82,16 +77,22 @@ class PDFGenerator : public Okular::Generator
 
         // [INHERITED] reparse configuration
         bool reparseConfig();
+        void addPages( KConfigDialog * );
 
         // [INHERITED] text exporting
         Okular::ExportFormat::List exportFormats() const;
         bool exportTo( const QString &fileName, const Okular::ExportFormat &format );
 
+    protected:
+        Okular::TextPage* textPage( Okular::Page *page );
+
     private slots:
         // (async related) receive data from the generator thread
         void threadFinished();
 
     private:
+        bool init(QVector<Okular::Page*> & pagesVector, const QString &walletKey);
+
         // friend class to access private document related variables
         friend class PDFPixmapGeneratorThread;
 
@@ -101,6 +102,8 @@ class PDFGenerator : public Okular::Generator
         void addAnnotations( Poppler::Page * popplerPage, Okular::Page * page );
         // fetch the transition information and add it to the page
         void addTransition( Poppler::Page * popplerPage, Okular::Page * page );
+        // load the source references from a pdfsync file
+        void loadPdfSync( const QString & fileName, QVector<Okular::Page*> & pagesVector );
 
         Okular::TextPage * abstractTextPage(const QList<Poppler::TextBox*> &text, double height, double width, int rot);
 
@@ -122,6 +125,8 @@ class PDFGenerator : public Okular::Generator
         Okular::DocumentFonts docFonts;
         mutable bool docEmbeddedFilesDirty;
         mutable QList<Okular::EmbeddedFile*> docEmbeddedFiles;
+
+        QVector<bool> rectsGenerated;
 };
 
 
