@@ -10,9 +10,7 @@
 #ifndef _OKULAR_GENERATOR_TIFF_H_
 #define _OKULAR_GENERATOR_TIFF_H_
 
-#include "core/generator.h"
-
-class TIFFGeneratorThread;
+#include <core/generator.h>
 
 class TIFFGenerator : public Okular::Generator
 {
@@ -24,25 +22,19 @@ class TIFFGenerator : public Okular::Generator
         bool loadDocument( const QString & fileName, QVector<Okular::Page*> & pagesVector );
         bool closeDocument();
 
-        bool canGeneratePixmap( bool async ) const;
-        void generatePixmap( Okular::PixmapRequest * request );
-
         const Okular::DocumentInfo * generateDocumentInfo();
 
-        bool supportsRotation() const { return true; };
+        bool print( KPrinter& printer );
 
-    private slots:
-        void slotThreadFinished();
+    protected:
+        QImage image( Okular::PixmapRequest * request );
 
     private:
         class Private;
         Private * const d;
 
-        TIFFGeneratorThread* thread;
+        void loadPages( QVector<Okular::Page*> & pagesVector );
 
-        void loadPages( QVector<Okular::Page*> & pagesVector, int rotation );
-
-        bool ready;
         Okular::DocumentInfo * m_docInfo;
 };
 
