@@ -16,17 +16,13 @@
 #ifndef _OKULAR_SHELL_H_
 #define _OKULAR_SHELL_H_
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include <kparts/mainwindow.h>
-#include <kmimetype.h>
 
+class KCmdLineArgs;
 class KRecentFilesAction;
-class K3TempFile;
 class KToggleAction;
 
+class KDocumentViewer;
 class Part;
 
 /**
@@ -43,14 +39,9 @@ class Shell : public KParts::MainWindow
 
 public:
   /**
-   * Default Constructor
+   * Constructor
    */
-  Shell();
-
-  /**
-   * Open an url
-   */
-  Shell(const KUrl &url);
+  Shell(KCmdLineArgs* args = 0, const KUrl &url = KUrl());
 
   /**
    * Default Destructor
@@ -84,7 +75,7 @@ private slots:
   void slotUpdateFullScreen();
   void slotShowMenubar();
 
-  void openUrl( const KUrl & url );
+  void openUrl( const KUrl & url, uint page = 0 );
   void delayedOpen();
 
 signals:
@@ -95,14 +86,14 @@ private:
   void setupAccel();
   void setupActions();
   void init();
-  bool handleCompressed(KUrl & url, const QString &path, const KMimeType::Ptr mimetype);
-  QStringList* fileFormats();
+  QStringList fileFormats() const;
 
 private:
+  KCmdLineArgs* m_args;
   KParts::ReadOnlyPart* m_part;
-  K3TempFile* m_tempfile;
+  KDocumentViewer* m_doc;
   KRecentFilesAction* m_recent;
-  QStringList* m_fileformats;
+  QStringList m_fileformats;
   KAction* m_printAction;
   KToggleAction* m_fullScreenAction;
   KToggleAction* m_showMenuBarAction;
