@@ -35,6 +35,8 @@ class NewStuffDialog : public QDialog
 
         // remove an already installed item
         void removeItem( AvailableItem * item );
+	
+	void loadItem( const QString & fileName );
 
     signals:
         // tells that a file has been installed
@@ -42,6 +44,8 @@ class NewStuffDialog : public QDialog
         // tells that a file has been removed
         void removedFile( QString oldFileName );
 
+	void loadItemClicked( const KUrl & fileName );
+
     private:
         // private storage class
         class NewStuffDialogPrivate * d;
