@@ -10,16 +10,15 @@
 // qt / kde includes
 #include <qrect.h>
 #include <qpainter.h>
+#include <qpalette.h>
 #include <qpixmap.h>
 #include <qimage.h>
 #include <qvarlengtharray.h>
-#include <qapplication.h>
-#include <kglobal.h>
 #include <kimageeffect.h>
 #include <kiconloader.h>
 #include <kstaticdeleter.h>
 #include <kdebug.h>
-
+#include <QApplication>
 // system includes
 #include <math.h>
 
@@ -28,6 +27,7 @@
 #include "core/area.h"
 #include "core/page.h"
 #include "core/annotations.h"
+#include "core/utils.h"
 #include "settings.h"
 
 static KStaticDeleter<QPixmap> sd;
@@ -36,27 +36,28 @@ QPixmap * busyPixmap = 0;
 #define TEXTANNOTATION_ICONSIZE 24
 
 void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page * page,
-    int pixID, int flags, int scaledWidth, int scaledHeight, const QRect & limits )
+    int pixID, int flags, int scaledWidth, int scaledHeight, const QRect &limits )
 {
     /** 1 - RETRIEVE THE 'PAGE+ID' PIXMAP OR A SIMILAR 'PAGE' ONE **/
     const QPixmap * pixmap = 0;
 
     // if a pixmap is present for given id, use it
-    if ( page->m_pixmaps.contains( pixID ) )
-        pixmap = page->m_pixmaps[ pixID ];
+    QMap< int, Okular::Page::PixmapObject >::const_iterator itPixmap = page->m_pixmaps.find( pixID );
+    if ( itPixmap != page->m_pixmaps.end() )
+        pixmap = itPixmap.value().m_pixmap;
 
     // else find the closest match using pixmaps of other IDs (great optim!)
     else if ( !page->m_pixmaps.isEmpty() )
     {
         int minDistance = -1;
-        QMap< int,QPixmap * >::const_iterator it = page->m_pixmaps.begin(), end = page->m_pixmaps.end();
+        QMap< int, Okular::Page::PixmapObject >::const_iterator it = page->m_pixmaps.begin(), end = page->m_pixmaps.end();
         for ( ; it != end; ++it )
         {
-            int pixWidth = (*it)->width(),
+            int pixWidth = (*it).m_pixmap->width(),
                 distance = pixWidth > scaledWidth ? pixWidth - scaledWidth : scaledWidth - pixWidth;
             if ( minDistance == -1 || distance < minDistance )
             {
-                pixmap = *it;
+                pixmap = (*it).m_pixmap;
                 minDistance = distance;
             }
         }
@@ -77,7 +78,7 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
         if ( !busyPixmap )
         {
             sd.setObject(busyPixmap, new QPixmap());
-            *busyPixmap = KGlobal::iconLoader()->loadIcon("okular", K3Icon::NoGroup, 32, K3Icon::DefaultState, 0, true);
+            *busyPixmap = KIconLoader::global()->loadIcon("okular", K3Icon::NoGroup, 32, K3Icon::DefaultState, 0, true);
         }
         // draw something on the blank page: the okular icon or a cross (as a fallback)
         if ( busyPixmap && !busyPixmap->isNull() )
@@ -102,7 +103,7 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
     // vectors containing objects to draw
     // make this a qcolor, rect map, since we don't need
     // to know s_id here! we are only drawing this right?
-    QList< QPair<QColor, Okular::NormalizedRect *> > * bufferedHighlights = 0;
+    QList< QPair<QColor, Okular::NormalizedRect> > * bufferedHighlights = 0;
     QList< Okular::Annotation * > * bufferedAnnotations = 0;
     QList< Okular::Annotation * > * unbufferedAnnotations = 0;
     // fill up lists with visible annotation/highlight objects/text selections
@@ -117,17 +118,17 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
         if ( canDrawHighlights )
         {
             if ( !bufferedHighlights )
-                 bufferedHighlights = new QList< QPair<QColor, Okular::NormalizedRect *>  >();
+                 bufferedHighlights = new QList< QPair<QColor, Okular::NormalizedRect> >();
 /*            else
             {*/
                 
                 Okular::NormalizedRect* limitRect = new Okular::NormalizedRect(nXMin, nYMin, nXMax, nYMax );
                 QLinkedList< Okular::HighlightAreaRect * >::const_iterator h2It = page->m_highlights.begin(), hEnd = page->m_highlights.end();
-                QList< Okular::NormalizedRect * >::const_iterator hIt;
+                Okular::HighlightAreaRect::const_iterator hIt;
                 for ( ; h2It != hEnd; ++h2It )
                     for (hIt=(*h2It)->begin(); hIt!=(*h2It)->end(); ++hIt)
                     {
-                        if ((*hIt)->intersects(limitRect))
+                        if ((*hIt).intersects(limitRect))
                             bufferedHighlights->append( qMakePair((*h2It)->color,*hIt) );
                     }
                 delete limitRect;
@@ -136,14 +137,14 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
         if ( canDrawTextSelection )
         {
             if ( !bufferedHighlights )
-                 bufferedHighlights = new QList< QPair<QColor, Okular::NormalizedRect *>  >();
+                 bufferedHighlights = new QList< QPair<QColor, Okular::NormalizedRect>  >();
 /*            else
             {*/
                 Okular::NormalizedRect* limitRect = new Okular::NormalizedRect(nXMin, nYMin, nXMax, nYMax );
-                QList< Okular::NormalizedRect * >::const_iterator hIt = page->m_textSelections->begin(), hEnd = page->m_textSelections->end();
+                Okular::HighlightAreaRect::const_iterator hIt = page->m_textSelections->begin(), hEnd = page->m_textSelections->end();
                 for ( ; hIt != hEnd; ++hIt )
                 {
-                    if ( (*hIt)->intersects( limitRect ) )
+                    if ( (*hIt).intersects( limitRect ) )
                         bufferedHighlights->append( qMakePair( page->m_textSelections->color, *hIt ) );
                 }
                 delete limitRect;
@@ -156,16 +157,19 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
             for ( ; aIt != aEnd; ++aIt )
             {
                 Okular::Annotation * ann = *aIt;
-                if ( ann->flags & Okular::Annotation::Hidden )
+                if ( ann->flags() & Okular::Annotation::Hidden )
                     continue;
 
-                bool intersects = ann->boundary.intersects( nXMin, nYMin, nXMax, nYMax );
+                bool intersects = ann->transformedBoundingRectangle().intersects( nXMin, nYMin, nXMax, nYMax );
                 if ( ann->subType() == Okular::Annotation::AText )
                 {
                     Okular::TextAnnotation * ta = static_cast< Okular::TextAnnotation * >( ann );
-                    if ( ta->textType == Okular::TextAnnotation::Linked )
+                    if ( ta->textType() == Okular::TextAnnotation::Linked )
                     {
-                        Okular::NormalizedRect iconrect( ann->boundary.left, ann->boundary.top, ann->boundary.left + TEXTANNOTATION_ICONSIZE / page->width(), ann->boundary.top + TEXTANNOTATION_ICONSIZE / page->height() );
+                        Okular::NormalizedRect iconrect( ann->transformedBoundingRectangle().left,
+                                                         ann->transformedBoundingRectangle().top,
+                                                         ann->transformedBoundingRectangle().left + TEXTANNOTATION_ICONSIZE / page->width(),
+                                                         ann->transformedBoundingRectangle().top + TEXTANNOTATION_ICONSIZE / page->height() );
                         intersects = iconrect.intersects( nXMin, nYMin, nXMax, nYMax );
                     }
                 }
@@ -173,7 +177,7 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
                 {
                     Okular::Annotation::SubType type = ann->subType();
                     if ( type == Okular::Annotation::ALine || type == Okular::Annotation::AHighlight ||
-                         type == Okular::Annotation::AInk  /*|| (type == Annotation::AGeom && ann->style.opacity < 0.99)*/ )
+                         type == Okular::Annotation::AInk  /*|| (type == Annotation::AGeom && ann->style().opacity() < 0.99)*/ )
                     {
                         if ( !bufferedAnnotations )
                             bufferedAnnotations = new QList< Okular::Annotation * >();
@@ -203,6 +207,7 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
         // 4A.1. if size is ok, draw the page pixmap using painter
         if ( pixmap->width() == scaledWidth && pixmap->height() == scaledHeight )
             destPainter->drawPixmap( limits.topLeft(), *pixmap, limits );
+
         // else draw a scaled portion of the magnified pixmap
         else
         {
@@ -269,12 +274,12 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
         if ( bufferedHighlights )
         {
             // draw highlights that are inside the 'limits' paint region
-            QList< QPair<QColor, Okular::NormalizedRect *> >::const_iterator hIt = bufferedHighlights->begin(), hEnd = bufferedHighlights->end();
+            QList< QPair<QColor, Okular::NormalizedRect> >::const_iterator hIt = bufferedHighlights->begin(), hEnd = bufferedHighlights->end();
             for ( ; hIt != hEnd; ++hIt )
             {
-                Okular::NormalizedRect * r = (*hIt).second;
+                const Okular::NormalizedRect & r = (*hIt).second;
                 // find out the rect to highlight on pixmap
-                QRect highlightRect = r->geometry( scaledWidth, scaledHeight ).intersect( limits );
+                QRect highlightRect = r.geometry( scaledWidth, scaledHeight ).intersect( limits );
                 highlightRect.translate( -limits.left(), -limits.top() );
 
                 // highlight composition (product: highlight color * destcolor)
@@ -323,8 +328,9 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
 
                     NormalizedPath path;
                     // normalize page point to image
-                    QLinkedList<Okular::NormalizedPoint>::const_iterator it = la->linePoints.begin();
-                    QLinkedList<Okular::NormalizedPoint>::const_iterator itEnd = la->linePoints.end();
+                    const QLinkedList<Okular::NormalizedPoint> points = la->transformedLinePoints();
+                    QLinkedList<Okular::NormalizedPoint>::const_iterator it = points.begin();
+                    QLinkedList<Okular::NormalizedPoint>::const_iterator itEnd = points.end();
                     for ( ; it != itEnd; ++it )
                     {
                         Okular::NormalizedPoint point;
@@ -334,39 +340,41 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
                     }
 
                     // draw the line as normalized path into image
-                    drawShapeOnImage( backImage, path, la->lineClosed, QPen( a->style.color,a->style.width ), QBrush(), pageScale ,Multiply);
+                    drawShapeOnImage( backImage, path, la->lineClosed(),
+                                      QPen( a->style().color(), a->style().width() ),
+                                      QBrush(), pageScale ,Multiply);
 
-                    if ( path.count() == 2 && fabs( la->lineLeadingFwdPt ) > 0.1 )
+                    if ( path.count() == 2 && fabs( la->lineLeadingForwardPoint() ) > 0.1 )
                     {
-                        Okular::NormalizedPoint delta( la->linePoints.last().x - la->linePoints.first().x, la->linePoints.first().y - la->linePoints.last().y );
+                        Okular::NormalizedPoint delta( la->transformedLinePoints().last().x - la->transformedLinePoints().first().x, la->transformedLinePoints().first().y - la->transformedLinePoints().last().y );
                         double angle = atan2( delta.y, delta.x );
                         if ( delta.y < 0 )
                             angle += 2 * M_PI;
 
-                        int sign = la->lineLeadingFwdPt > 0.0 ? 1 : -1;
-                        double LLx = fabs( la->lineLeadingFwdPt ) * cos( angle + sign * M_PI_2 + 2 * M_PI ) / page->width();
-                        double LLy = fabs( la->lineLeadingFwdPt ) * sin( angle + sign * M_PI_2 + 2 * M_PI ) / page->height();
+                        int sign = la->lineLeadingForwardPoint() > 0.0 ? 1 : -1;
+                        double LLx = fabs( la->lineLeadingForwardPoint() ) * cos( angle + sign * M_PI_2 + 2 * M_PI ) / page->width();
+                        double LLy = fabs( la->lineLeadingForwardPoint() ) * sin( angle + sign * M_PI_2 + 2 * M_PI ) / page->height();
 
                         NormalizedPath path2;
                         NormalizedPath path3;
 
                         Okular::NormalizedPoint point;
-                        point.x = ( la->linePoints.first().x + LLx - xOffset ) * xScale;
-                        point.y = ( la->linePoints.first().y - LLy - yOffset ) * yScale;
+                        point.x = ( la->transformedLinePoints().first().x + LLx - xOffset ) * xScale;
+                        point.y = ( la->transformedLinePoints().first().y - LLy - yOffset ) * yScale;
                         path2.append( point );
-                        point.x = ( la->linePoints.last().x + LLx - xOffset ) * xScale;
-                        point.y = ( la->linePoints.last().y - LLy - yOffset ) * yScale;
+                        point.x = ( la->transformedLinePoints().last().x + LLx - xOffset ) * xScale;
+                        point.y = ( la->transformedLinePoints().last().y - LLy - yOffset ) * yScale;
                         path3.append( point );
                         // do we have the extension on the "back"?
-                        if ( fabs( la->lineLeadingBackPt ) > 0.1 )
+                        if ( fabs( la->lineLeadingBackwardPoint() ) > 0.1 )
                         {
-                            double LLEx = la->lineLeadingBackPt * cos( angle - sign * M_PI_2 + 2 * M_PI ) / page->width();
-                            double LLEy = la->lineLeadingBackPt * sin( angle - sign * M_PI_2 + 2 * M_PI ) / page->height();
-                            point.x = ( la->linePoints.first().x + LLEx - xOffset ) * xScale;
-                            point.y = ( la->linePoints.first().y - LLEy - yOffset ) * yScale;
+                            double LLEx = la->lineLeadingBackwardPoint() * cos( angle - sign * M_PI_2 + 2 * M_PI ) / page->width();
+                            double LLEy = la->lineLeadingBackwardPoint() * sin( angle - sign * M_PI_2 + 2 * M_PI ) / page->height();
+                            point.x = ( la->transformedLinePoints().first().x + LLEx - xOffset ) * xScale;
+                            point.y = ( la->transformedLinePoints().first().y - LLEy - yOffset ) * yScale;
                             path2.append( point );
-                            point.x = ( la->linePoints.last().x + LLEx - xOffset ) * xScale;
-                            point.y = ( la->linePoints.last().y - LLEy - yOffset ) * yScale;
+                            point.x = ( la->transformedLinePoints().last().x + LLEx - xOffset ) * xScale;
+                            point.y = ( la->transformedLinePoints().last().y - LLEy - yOffset ) * yScale;
                             path3.append( point );
                         }
                         else
@@ -375,8 +383,8 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
                             path3.append( path[1] );
                         }
 
-                        drawShapeOnImage( backImage, path2, false, QPen( a->style.color, a->style.width ), QBrush(), pageScale, Multiply );
-                        drawShapeOnImage( backImage, path3, false, QPen( a->style.color, a->style.width ), QBrush(), pageScale, Multiply );
+                        drawShapeOnImage( backImage, path2, false, QPen( a->style().color(), a->style().width() ), QBrush(), pageScale, Multiply );
+                        drawShapeOnImage( backImage, path3, false, QPen( a->style().color(), a->style().width() ), QBrush(), pageScale, Multiply );
                     }
                 }
                 // draw GeomAnnotation MISSING: all
@@ -389,20 +397,20 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
                 {
                     // get the annotation
                     Okular::HighlightAnnotation * ha = (Okular::HighlightAnnotation *) a;
-                    Okular::HighlightAnnotation::HighlightType type = ha->highlightType;
+                    Okular::HighlightAnnotation::HighlightType type = ha->highlightType();
 
                     // draw each quad of the annotation
-                    int quads = ha->highlightQuads.size();
+                    int quads = ha->highlightQuads().size();
                     for ( int q = 0; q < quads; q++ )
                     {
                         NormalizedPath path;
-                        const Okular::HighlightAnnotation::Quad & quad = ha->highlightQuads[ q ];
+                        const Okular::HighlightAnnotation::Quad & quad = ha->highlightQuads()[ q ];
                         // normalize page point to image
                         for ( int i = 0; i < 4; i++ )
                         {
                             Okular::NormalizedPoint point;
-                            point.x = (quad.points[ i ].x - xOffset) * xScale;
-                            point.y = (quad.points[ i ].y - yOffset) * yScale;
+                            point.x = (quad.transformedPoint( i ).x - xOffset) * xScale;
+                            point.y = (quad.transformedPoint( i ).y - yOffset) * yScale;
                             path.append( point );
                         }
                         // draw the normalized path into image
@@ -410,7 +418,7 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
                         {
                             // highlight the whole rect
                             case Okular::HighlightAnnotation::Highlight:
-                                drawShapeOnImage( backImage, path, true, QPen(), a->style.color, pageScale, Multiply );
+                                drawShapeOnImage( backImage, path, true, QPen(), a->style().color(), pageScale, Multiply );
                                 break;
                             // highlight the bottom part of the rect
                             case Okular::HighlightAnnotation::Squiggly:
@@ -418,7 +426,7 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
                                 path[ 3 ].y = ( path[ 0 ].y + path[ 3 ].y ) / 2.0;
                                 path[ 2 ].x = ( path[ 1 ].x + path[ 2 ].x ) / 2.0;
                                 path[ 2 ].y = ( path[ 1 ].y + path[ 2 ].y ) / 2.0;
-                                drawShapeOnImage( backImage, path, true, QPen(), a->style.color, pageScale, Multiply );
+                                drawShapeOnImage( backImage, path, true, QPen(), a->style().color(), pageScale, Multiply );
                                 break;
                             // make a line at 3/4 of the height
                             case Okular::HighlightAnnotation::Underline:
@@ -428,7 +436,7 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
                                 path[ 1 ].y = ( 3 * path[ 1 ].y + path[ 2 ].y ) / 4.0;
                                 path.pop_back();
                                 path.pop_back();
-                                drawShapeOnImage( backImage, path, false, QPen( a->style.color, 2 ), QBrush(), pageScale );
+                                drawShapeOnImage( backImage, path, false, QPen( a->style().color(), 2 ), QBrush(), pageScale );
                                 break;
                             // make a line at 1/2 of the height
                             case Okular::HighlightAnnotation::StrikeOut:
@@ -438,7 +446,7 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
                                 path[ 1 ].y = ( path[ 1 ].y + path[ 2 ].y ) / 2.0;
                                 path.pop_back();
                                 path.pop_back();
-                                drawShapeOnImage( backImage, path, false, QPen( a->style.color, 2 ), QBrush(), pageScale );
+                                drawShapeOnImage( backImage, path, false, QPen( a->style().color(), 2 ), QBrush(), pageScale );
                                 break;
                         }
                     }
@@ -450,11 +458,13 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
                     Okular::InkAnnotation * ia = (Okular::InkAnnotation *) a;
 
                     // draw each ink path
-                    int paths = ia->inkPaths.size();
+                    const QList< QLinkedList<Okular::NormalizedPoint> > transformedInkPaths = ia->transformedInkPaths();
+
+                    int paths = transformedInkPaths.size();
                     for ( int p = 0; p < paths; p++ )
                     {
                         NormalizedPath path;
-                        const QLinkedList<Okular::NormalizedPoint> & inkPath = ia->inkPaths[ p ];
+                        const QLinkedList<Okular::NormalizedPoint> & inkPath = transformedInkPaths[ p ];
 
                         // normalize page point to image
                         QLinkedList<Okular::NormalizedPoint>::const_iterator pIt = inkPath.begin(), pEnd = inkPath.end();
@@ -467,7 +477,8 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
                             path.append( point );
                         }
                         // draw the normalized path into image
-                        drawShapeOnImage( backImage, path, false, QPen( a->style.color, a->style.width ), QBrush(), pageScale );
+                        QColor acolor( a->style().color().red(), a->style().color().green(), a->style().color().blue(), (int)( 255.0 * a->style().opacity() ) );
+                        drawShapeOnImage( backImage, path, false, QPen( acolor, a->style().width() ), QBrush(), pageScale );
                     }
                 }
             } // end current annotation drawing
@@ -491,12 +502,14 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
             Okular::Annotation * a = *aIt;
 
             // honor opacity settings on supported types
-            unsigned int opacity = (unsigned int)( 255.0 * a->style.opacity );
-            if ( opacity <= 0 )
+            unsigned int opacity = (unsigned int)( 255.0 * a->style().opacity() );
+            // skip the annotation drawing if all the annotation is fully
+            // transparent, but not with text annotations
+            if ( opacity <= 0 && a->subType() != Okular::Annotation::AText )
                 continue;
 
             // get annotation boundary and drawn rect
-            QRect annotBoundary = a->boundary.geometry( scaledWidth, scaledHeight );
+            QRect annotBoundary = a->transformedBoundingRectangle().geometry( scaledWidth, scaledHeight );
             QRect annotRect = annotBoundary.intersect( limits );
             QRect innerRect( annotRect.left() - annotBoundary.left(), annotRect.top() -
                     annotBoundary.top(), annotRect.width(), annotRect.height() );
@@ -507,9 +520,9 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
             if ( type == Okular::Annotation::AText )
             {
                 Okular::TextAnnotation * text = (Okular::TextAnnotation *)a;
-                if ( text->textType == Okular::TextAnnotation::InPlace )
+                if ( text->textType() == Okular::TextAnnotation::InPlace )
                 {
-                    QRect bigRect = a->boundary.geometry( (int)page->width(), (int)page->height() );
+                    QRect bigRect = a->transformedBoundingRectangle().geometry( (int)page->width(), (int)page->height() );
 
                     // the strategy behind 'bigger': if where are we going to
                     // draw is bigger than the Page, then draw the rect only
@@ -517,14 +530,14 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
                     // otherwise draw it right after the text
                     bool bigger = mixedPainter->device()->width() > page->width();
                     QImage image( bigRect.size(), QImage::Format_ARGB32 );
-                    image.fill( qRgba( a->style.color.red(), a->style.color.green(), a->style.color.blue(), 255 ) );
+                    image.fill( qRgba( a->style().color().red(), a->style().color().green(), a->style().color().blue(), opacity ) );
                     QPainter painter( &image );
                     painter.setPen( Qt::black );
-                    painter.setFont( text->textFont );
-                    Qt::AlignmentFlag halign = ( text->inplaceAlign == 1 ? Qt::AlignHCenter : ( text->inplaceAlign == 2 ? Qt::AlignRight : Qt::AlignLeft ) );
+                    painter.setFont( text->textFont() );
+                    Qt::AlignmentFlag halign = ( text->inplaceAlignment() == 1 ? Qt::AlignHCenter : ( text->inplaceAlignment() == 2 ? Qt::AlignRight : Qt::AlignLeft ) );
                     painter.drawText( 2, 2, image.width() - 2, image.height() - 2,
                                       Qt::AlignTop | halign | Qt::TextWordWrap,
-                                      text->inplaceText );
+                                      text->inplaceText() );
                     if ( !bigger )
                         painter.drawRect( 0, 0, image.width() - 1, image.height() - 1 );
                     painter.end();
@@ -540,13 +553,13 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
 //                    mixedPainter->drawImage( annotBoundary.topLeft(), image.scaled( annotBoundary.size(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation ) );
                     mixedPainter->drawImage( annotBoundary.topLeft(), image );
                 }
-                else if ( text->textType == Okular::TextAnnotation::Linked )
+                else if ( text->textType() == Okular::TextAnnotation::Linked )
                 {
                 // get pixmap, colorize and alpha-blend it
                     QString path;
-                    QPixmap pixmap = KGlobal::iconLoader()->loadIcon( text->textIcon.toLower(), K3Icon::User, 32, K3Icon::DefaultState, &path, true );
+                    QPixmap pixmap = KIconLoader::global()->loadIcon( text->textIcon().toLower(), K3Icon::User, 32, K3Icon::DefaultState, &path, true );
                     if ( path.isEmpty() )
-                        pixmap = KGlobal::iconLoader()->loadIcon( text->textIcon.toLower(), K3Icon::NoGroup, 32 );
+                        pixmap = KIconLoader::global()->loadIcon( text->textIcon().toLower(), K3Icon::NoGroup, 32 );
                     QImage scaledImage;
                     QRect annotBoundary2 = QRect( annotBoundary.topLeft(), QSize( TEXTANNOTATION_ICONSIZE, TEXTANNOTATION_ICONSIZE ) );
                     QRect annotRect2 = annotBoundary2.intersect( limits );
@@ -558,8 +571,8 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
                     // if the annotation color is valid (ie it was set), then
                     // use it to colorize the icon, otherwise the icon will be
                     // "gray"
-                    if ( a->style.color.isValid() )
-                        colorizeImage( scaledImage, a->style.color, opacity );
+                    if ( a->style().color().isValid() )
+                        colorizeImage( scaledImage, a->style().color(), opacity );
                     pixmap = QPixmap::fromImage( scaledImage );
 
                 // draw the mangled image to painter
@@ -574,9 +587,9 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
 
                 // get pixmap and alpha blend it if needed
                 QString path;
-                QPixmap pixmap = KGlobal::iconLoader()->loadIcon( stamp->stampIconName.toLower(), K3Icon::User, qMin( annotBoundary.width(), annotBoundary.height() ), K3Icon::DefaultState, &path, true );
+                QPixmap pixmap = KIconLoader::global()->loadIcon( stamp->stampIconName().toLower(), K3Icon::User, qMin( annotBoundary.width(), annotBoundary.height() ), K3Icon::DefaultState, &path, true );
                 if ( path.isEmpty() )
-                    pixmap = KGlobal::iconLoader()->loadIcon( stamp->stampIconName.toLower(), K3Icon::NoGroup, qMin( annotBoundary.width(), annotBoundary.height() ) );
+                    pixmap = KIconLoader::global()->loadIcon( stamp->stampIconName().toLower(), K3Icon::NoGroup, qMin( annotBoundary.width(), annotBoundary.height() ) );
                 QImage scaledImage;
                 scalePixmapOnImage( scaledImage, &pixmap, annotBoundary.width(),
                                     annotBoundary.height(), innerRect, QImage::Format_ARGB32 );
@@ -588,6 +601,36 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
                 mixedPainter->drawPixmap( annotRect.topLeft(), pixmap );
             }
             // draw GeomAnnotation
+            else if ( type == Okular::Annotation::AGeom )
+            {
+                Okular::GeomAnnotation * geom = (Okular::GeomAnnotation *)a;
+                QImage shape( annotBoundary.size(), QImage::Format_ARGB32 );
+                shape.fill( qRgba( 0, 0, 0, 0 ) );
+                // width is already divided by two
+                double width = geom->geometricalPointWidth() * Okular::Utils::dpiX() / ( 72.0 * 2.0 ) * scaledWidth / page->width();
+                QPainter p( &shape );
+                p.setPen( QPen( QBrush( QColor( a->style().color().red(), a->style().color().green(), a->style().color().blue(), opacity ) ), width * 2, Qt::SolidLine, Qt::SquareCap, Qt::MiterJoin ) );
+                QRectF r( .0, .0, annotBoundary.width(), annotBoundary.height() );
+                r.adjust( width, width, -width, -width );
+                if ( geom->geometricalType() == Okular::GeomAnnotation::InscribedSquare )
+                    p.drawRect( r );
+                else
+                    p.drawEllipse( r );
+                if ( geom->geometricalInnerColor().isValid() )
+                {
+                    p.setPen( Qt::NoPen );
+                    const QColor color = geom->geometricalInnerColor();
+                    p.setBrush( QColor( color.red(), color.green(), color.blue(), opacity ) );
+                    r.adjust( width, width, -width, -width );
+                    if ( geom->geometricalType() == Okular::GeomAnnotation::InscribedSquare )
+                        p.drawRect( r );
+                    else
+                        p.drawEllipse( r );
+                }
+                p.end();
+                mixedPainter->drawImage( annotBoundary.topLeft(), shape );
+            }
+#if 0
             else // WARNING: TEMPORARY CODE! migrate everything to AGG
             {
                 //GeomAnnotation * geom = (GeomAnnotation *)a;
@@ -601,11 +644,12 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
                 //}
                 //else if ( geom->geomType == GeomAnnotation::InscribedCircle )
             }
+#endif
 
             // draw extents rectangle
             if ( Okular::Settings::debugDrawAnnotationRect() )
             {
-                mixedPainter->setPen( a->style.color );
+                mixedPainter->setPen( a->style().color() );
                 mixedPainter->drawRect( annotBoundary );
             }
         }
@@ -841,7 +885,7 @@ void PagePainter::drawShapeOnImage(
     if ( brush.style() != Qt::NoBrush )
     {
         const QColor & brushColor = brush.color();
-        render.color( agg::rgba8( brushColor.red(), brushColor.green(), brushColor.blue() ) );
+        render.color( agg::rgba8( brushColor.red(), brushColor.green(), brushColor.blue(), (int)( agg::rgba8::base_mask * brushColor.alphaF() ) ) );
         rasterizer.add_path( path );
         agg::render_scanlines( rasterizer, scanline, render );
         rasterizer.reset();
@@ -852,7 +896,7 @@ void PagePainter::drawShapeOnImage(
     if ( penWidth > 0.1 )
     {
         const QColor & penColor = pen.color();
-        render.color( agg::rgba8( penColor.red(), penColor.green(), penColor.blue() ) );
+        render.color( agg::rgba8( penColor.red(), penColor.green(), penColor.blue(), (int)( agg::rgba8::base_mask * penColor.alphaF() ) ) );
 #if 0
         // BSPLINE curve over path
         typedef agg::conv_bspline< agg::path_storage > conv_bspline_type;
