@@ -51,9 +51,6 @@ Q_OBJECT
         PageView( QWidget *parent, Okular::Document *document );
         ~PageView();
 
-        //set pop-up annotation window states,such as hide/open ,position...
-        void setAnnotsWindow(Okular::Annotation * annot);
-
         // Zoom mode ( last 4 are internally used only! )
         enum ZoomMode { ZoomFixed = 0, ZoomFitWidth = 1, ZoomFitPage = 2, ZoomFitText,
                         ZoomIn, ZoomOut, ZoomRefreshCurrent };
@@ -63,7 +60,7 @@ Q_OBJECT
         void setupActions( KActionCollection * collection );
 
         // misc methods (from RMB menu/children)
-        bool canFitPageWidth();
+        bool canFitPageWidth() const;
         void fitPageWidth( int page );
         // keep in sync with pageviewutils
         void displayMessage( const QString & message , PageViewMessage::Icon icon=PageViewMessage::Info, int duration=-1 );
@@ -74,7 +71,7 @@ Q_OBJECT
         void notifyViewportChanged( bool smoothMove );
         void notifyPageChanged( int pageNumber, int changedFlags );
         void notifyContentsCleared( int changedFlags );
-        bool canUnloadPixmap( int pageNum );
+        bool canUnloadPixmap( int pageNum ) const;
 
         QList< Okular::RegularAreaRect * > textSelections( const QPoint& start, const QPoint& end, int& firstpage );
         Okular::RegularAreaRect * textSelectionForItem( PageViewItem * item, const QPoint & startPoint = QPoint(), const QPoint & endPoint = QPoint() );
@@ -97,6 +94,12 @@ Q_OBJECT
             displayMessage( message, PageViewMessage::Warning, duration );
         }
 
+        void copyTextSelection() const;
+
+        void setAnnotationWindow( Okular::Annotation *annotation );
+
+        void removeAnnotationWindow( Okular::Annotation *annotation );
+
     signals:
         void urlDropped( const KUrl& );
         void rightClick( const Okular::Page *, const QPoint & );
@@ -157,9 +160,11 @@ Q_OBJECT
         void slotMoveViewport();
         // activated by the autoscroll timer (Shift+Up/Down keys)
         void slotAutoScoll();
+        // activated by the dragScroll timer
+        void slotDragScroll();
         // type-ahead find timeout
         void slotStopFindAhead();
-        // sow the welcome message
+        // show the welcome message
         void slotShowWelcome();
 
         // connected to local actions (toolbar, menu, ..)
@@ -178,6 +183,9 @@ Q_OBJECT
         void slotToggleAnnotator( bool );
         void slotScrollUp();
         void slotScrollDown();
+        void slotRotateClockwise();
+        void slotRotateCounterClockwise();
+        void slotRotateOriginal();
 };
 
 #endif
