@@ -42,15 +42,14 @@ class PickPointEngine : public AnnotatorEngine
 {
     public:
         PickPointEngine( const QDomElement & engineElement )
-    : AnnotatorEngine( engineElement ), clicked( false ), xscale(1.0), yscale(1.0)
+            : AnnotatorEngine( engineElement ), clicked( false ), pixmap( 0 ),
+              xscale( 1.0 ), yscale( 1.0 )
         {
             // parse engine specific attributes
             pixmapName = engineElement.attribute( "hoverIcon" );
             QString stampname = m_annotElement.attribute( "icon" );
             if ( m_annotElement.attribute( "type" ) == "Stamp" && !stampname.simplified().isEmpty() )
                 pixmapName = stampname;
-            if ( pixmapName.simplified().isEmpty() )
-                pixmapName = "okular";
             center = QVariant( engineElement.attribute( "center" ) ).toBool();
             bool ok = true;
             size = engineElement.attribute( "size", "32" ).toInt( &ok );
@@ -58,7 +57,8 @@ class PickPointEngine : public AnnotatorEngine
                 size = 32;
 
             // create engine objects
-            pixmap = new QPixmap( DesktopIcon( pixmapName, size ) );
+            if ( !pixmapName.simplified().isEmpty() )
+                pixmap = new QPixmap( DesktopIcon( pixmapName, size ) );
         }
 
         ~PickPointEngine()
@@ -98,7 +98,7 @@ class PickPointEngine : public AnnotatorEngine
             // update variables and extents (zoom invariant rect)
             point.x = nX;
             point.y = nY;
-            if ( center )
+            if ( center && pixmap )
             {
                 rect.left = nX - ( pixmap->width() / ( xScale * 2.0 ) );
                 rect.top = nY - ( pixmap->height() / ( yScale * 2.0 ) );
@@ -108,8 +108,8 @@ class PickPointEngine : public AnnotatorEngine
                 rect.left = nX;
                 rect.top = nY;
             }
-            rect.right = rect.left + ( pixmap->width() / xScale );
-            rect.bottom = rect.top + ( pixmap->height() / yScale );
+            rect.right = rect.left + ( pixmap ? pixmap->width() / xScale : 0 );
+            rect.bottom = rect.top + ( pixmap ? pixmap->height() / yScale : 0 );
             return rect.geometry( (int)xScale, (int)yScale ).adjusted( 0, 0, 1, 1 );
         }
 
@@ -128,7 +128,7 @@ class PickPointEngine : public AnnotatorEngine
             // find out annotation's description node
             if ( m_annotElement.isNull() )
                 return QList< Okular::Annotation* >();
-            
+
             // find out annotation's type
             Okular::Annotation * ann = 0;
             QString typeString = m_annotElement.attribute( "type" );
@@ -144,8 +144,8 @@ class PickPointEngine : public AnnotatorEngine
                     //add note
                     Okular::TextAnnotation * ta = new Okular::TextAnnotation();
                     ann = ta;
-                    ta->inplaceText=note;
-                    ta->textType = Okular::TextAnnotation::InPlace;
+                    ta->setInplaceText( note );
+                    ta->setTextType( Okular::TextAnnotation::InPlace );
                     //set boundary
                     rect.left = qMin(startpoint.x,point.x);
                     rect.top = qMin(startpoint.y,point.y);
@@ -153,36 +153,36 @@ class PickPointEngine : public AnnotatorEngine
                     rect.bottom = qMax(startpoint.y,point.y);
                     kDebug()<<"astario:   xyScale="<<xscale<<","<<yscale<<endl;
                     static int padding = 2;
-                    QFontMetricsF mf(ta->textFont);
+                    QFontMetricsF mf(ta->textFont());
                     QRectF rcf = mf.boundingRect( Okular::NormalizedRect( rect.left, rect.top, 1.0, 1.0 ).geometry( (int)pagewidth, (int)pageheight ).adjusted( padding, padding, -padding, -padding ),
-                                                  Qt::AlignTop | Qt::AlignLeft | Qt::TextWordWrap, ta->inplaceText );
+                                                  Qt::AlignTop | Qt::AlignLeft | Qt::TextWordWrap, ta->inplaceText() );
                     rect.right = qMax(rect.right, rect.left+(rcf.width()+padding*2)/pagewidth);
                     rect.bottom = qMax(rect.bottom, rect.top+(rcf.height()+padding*2)/pageheight);
-                    ta->boundary=this->rect;
-                    ta->window.summary="TextBox";
+                    ta->setBoundingRectangle( this->rect );
+                    ta->window().setSummary( "TextBox" );
                 }
             }
             else if ( typeString == "Text")
             {
                 Okular::TextAnnotation * ta = new Okular::TextAnnotation();
                 ann = ta;
-                ta->textType = Okular::TextAnnotation::Linked;
-                ta->window.text="";
+                ta->setTextType( Okular::TextAnnotation::Linked );
+                ta->window().setText( QString() );
                 //ta->window.flags &= ~(Okular::Annotation::Hidden);
                 double iconhei=0.03;
                 rect.left = point.x;
                 rect.top = point.y;
                 rect.right=rect.left+iconhei;
                 rect.bottom=rect.top+iconhei*xscale/yscale;
-                ta->boundary=this->rect;
-                ta->window.summary="Note";
+                ta->setBoundingRectangle( this->rect );
+                ta->window().setSummary( "Note" );
             }
             // create StampAnnotation from path
             else if ( typeString == "Stamp" )
             {
                 Okular::StampAnnotation * sa = new Okular::StampAnnotation();
                 ann = sa;
-                sa->stampIconName = pixmapName;
+                sa->setStampIconName( pixmapName );
                 double stampxscale = size / xscale;
                 double stampyscale = size / yscale;
                 if ( center )
@@ -197,7 +197,25 @@ class PickPointEngine : public AnnotatorEngine
                 }
                 rect.right = rect.left + stampxscale;
                 rect.bottom = rect.top + stampyscale;
-                sa->boundary = rect;
+                sa->setBoundingRectangle( rect );
+            }
+            // create GeomAnnotation
+            else if ( typeString == "GeomSquare" || typeString == "GeomCircle" )
+            {
+                Okular::GeomAnnotation * ga = new Okular::GeomAnnotation();
+                ann = ga;
+                // set the type
+                if ( typeString == "GeomSquare" )
+                    ga->setGeometricalType( Okular::GeomAnnotation::InscribedSquare );
+                else
+                    ga->setGeometricalType( Okular::GeomAnnotation::InscribedCircle );
+                ga->setGeometricalPointWidth( 18 );
+                //set boundary
+                rect.left = qMin( startpoint.x, point.x );
+                rect.top = qMin( startpoint.y, point.y );
+                rect.right = qMax( startpoint.x, point.x );
+                rect.bottom = qMax( startpoint.y, point.y );
+                ga->setBoundingRectangle( rect );
             }
 
             // safety check
@@ -205,10 +223,10 @@ class PickPointEngine : public AnnotatorEngine
                 return QList< Okular::Annotation* >();
 
             // set common attributes
-            ann->style.color = m_annotElement.hasAttribute( "color" ) ?
-                    m_annotElement.attribute( "color" ) : m_engineColor;
+            ann->style().setColor( m_annotElement.hasAttribute( "color" ) ?
+                    m_annotElement.attribute( "color" ) : m_engineColor );
             if ( m_annotElement.hasAttribute( "opacity" ) )
-                ann->style.opacity = m_annotElement.attribute( "opacity", "1.0" ).toDouble();
+                ann->style().setOpacity( m_annotElement.attribute( "opacity", "1.0" ).toDouble() );
 
             // return annotation
             return QList< Okular::Annotation* >() << ann;
@@ -330,7 +348,7 @@ class PolyLineEngine : public AnnotatorEngine
         {
             m_creationCompleted = false;
 
-			// find out annotation's description node
+            // find out annotation's description node
             if ( m_annotElement.isNull() )
                 return QList< Okular::Annotation* >();
 
@@ -343,14 +361,20 @@ class PolyLineEngine : public AnnotatorEngine
             {
                 if ( points.count() < 2 )
                     return QList< Okular::Annotation* >();
+
                 //add note
                 Okular::LineAnnotation * la = new Okular::LineAnnotation();
                 ann = la;
+                QLinkedList<Okular::NormalizedPoint> list;
                 for ( int i = 0; i < points.count(); ++i )
-                    la->linePoints.append( points[i] );
+                    list.append( points[ i ] );
+
+                la->setLinePoints( list );
+
                 if ( numofpoints == -1 )
-                    la->lineClosed = true;
-                la->boundary = normRect;
+                    la->setLineClosed( true );
+
+                la->setBoundingRectangle( normRect );
 
             }
 
@@ -359,10 +383,10 @@ class PolyLineEngine : public AnnotatorEngine
                 return QList< Okular::Annotation* >();
 
             // set common attributes
-            ann->style.color = m_annotElement.hasAttribute( "color" ) ?
-                    m_annotElement.attribute( "color" ) : m_engineColor;
+            ann->style().setColor( m_annotElement.hasAttribute( "color" ) ?
+                    m_annotElement.attribute( "color" ) : m_engineColor );
             if ( m_annotElement.hasAttribute( "opacity" ) )
-                ann->style.opacity = m_annotElement.attribute( "opacity", "1.0" ).toDouble();
+                ann->style().setOpacity( m_annotElement.attribute( "opacity", "1.0" ).toDouble() );
             // return annotation
 
             return QList< Okular::Annotation* >() << ann;
@@ -412,24 +436,21 @@ class TextSelectorEngine : public AnnotatorEngine
                     QPoint start( (int)( lastPoint.x * item()->width() ), (int)( lastPoint.y * item()->height() ) );
                     QPoint end( (int)( nX * item()->width() ), (int)( nY * item()->height() ) );
                     Okular::RegularAreaRect * newselection = m_pageView->textSelectionForItem( item(), start, end );
-                    QList<QRect> * geom = newselection->geometry( (int)xScale, (int)yScale );
+                    QList<QRect> geom = newselection->geometry( (int)xScale, (int)yScale );
                     QRect newrect;
-                    if ( geom )
+                    foreach( const QRect& r, geom )
                     {
-                        foreach( QRect r, *geom )
-                        {
-                            if ( newrect.isNull() )
-                                newrect = r;
-                            else
-                                newrect |= r;
-                        }
+                        if ( newrect.isNull() )
+                            newrect = r;
+                        else
+                            newrect |= r;
                     }
                     rect |= newrect;
                     delete selection;
                     selection = newselection;
                 }
             }
-            else if ( type == Release )
+            else if ( type == Release && selection )
             {
                 m_creationCompleted = true;
             }
@@ -444,9 +465,9 @@ class TextSelectorEngine : public AnnotatorEngine
                 QColor col = m_engineColor;
                 col.setAlphaF( 0.5 );
                 painter->setBrush( col );
-                foreach( Okular::NormalizedRect * r, *selection )
+                foreach( const Okular::NormalizedRect & r, *selection )
                 {
-                    painter->drawRect( r->geometry( (int)xScale, (int)yScale ) );
+                    painter->drawRect( r.geometry( (int)xScale, (int)yScale ) );
                 }
             }
         }
@@ -455,8 +476,8 @@ class TextSelectorEngine : public AnnotatorEngine
         {
             m_creationCompleted = false;
 
-            // find out annotation's description node
-            if ( m_annotElement.isNull() )
+            // safety checks
+            if ( m_annotElement.isNull() || !selection )
                 return QList< Okular::Annotation* >();
 
             // find out annotation's type
@@ -489,19 +510,19 @@ class TextSelectorEngine : public AnnotatorEngine
             if ( typevalid )
             {
                 Okular::HighlightAnnotation * ha = new Okular::HighlightAnnotation();
-                ha->highlightType = type;
-                ha->boundary = Okular::NormalizedRect( rect, (int)item()->width(), (int)item()->height() );
-                foreach ( Okular::NormalizedRect * rect, *selection )
+                ha->setHighlightType( type );
+                ha->setBoundingRectangle( Okular::NormalizedRect( rect, (int)item()->width(), (int)item()->height() ) );
+                foreach ( const Okular::NormalizedRect & r, *selection )
                 {
                     Okular::HighlightAnnotation::Quad q;
-                    q.capStart = false;
-                    q.capEnd = false;
-                    q.feather = 1.0;
-                    q.points[0] = Okular::NormalizedPoint( rect->left, rect->bottom );
-                    q.points[1] = Okular::NormalizedPoint( rect->right, rect->bottom );
-                    q.points[2] = Okular::NormalizedPoint( rect->right, rect->top );
-                    q.points[3] = Okular::NormalizedPoint( rect->left, rect->top );
-                    ha->highlightQuads.append( q );
+                    q.setCapStart( false );
+                    q.setCapEnd( false );
+                    q.setFeather( 1.0 );
+                    q.setPoint( Okular::NormalizedPoint( r.left, r.bottom ), 0 );
+                    q.setPoint( Okular::NormalizedPoint( r.right, r.bottom ), 1 );
+                    q.setPoint( Okular::NormalizedPoint( r.right, r.top ), 2 );
+                    q.setPoint( Okular::NormalizedPoint( r.left, r.top ), 3 );
+                    ha->highlightQuads().append( q );
                 }
                 ann = ha;
             }
@@ -511,10 +532,10 @@ class TextSelectorEngine : public AnnotatorEngine
                 return QList< Okular::Annotation* >();
 
             // set common attributes
-            ann->style.color = m_annotElement.hasAttribute( "color" ) ?
-                m_annotElement.attribute( "color" ) : m_engineColor;
+            ann->style().setColor( m_annotElement.hasAttribute( "color" ) ?
+                m_annotElement.attribute( "color" ) : m_engineColor );
             if ( m_annotElement.hasAttribute( "opacity" ) )
-                ann->style.opacity = m_annotElement.attribute( "opacity", "1.0" ).toDouble();
+                ann->style().setOpacity( m_annotElement.attribute( "opacity", "1.0" ).toDouble() );
 
             // return annotations
             return QList< Okular::Annotation* >() << ann;
@@ -564,6 +585,8 @@ void PageViewAnnotator::setEnabled( bool on )
         if ( m_toolBar )
             m_toolBar->hideAndDestroy();
         m_toolBar = 0;
+        // deactivate the active tool, if any
+        slotToolSelected( -1 );
         return;
     }
 
@@ -605,7 +628,7 @@ void PageViewAnnotator::setEnabled( bool on )
     m_toolBar->showItems( (PageViewToolBar::Side)Okular::Settings::editToolBarPlacement(), items );
 
     // ask for Author's name if not already set
-    if ( Okular::Settings::annotationsAuthor().isEmpty() )
+    if ( Okular::Settings::identityAuthor().isEmpty() )
     {
         // get default username from the kdelibs/kdecore/KUser
         KUser currentUser;
@@ -620,7 +643,7 @@ void PageViewAnnotator::setEnabled( bool on )
             firstTry = false;
         }
         // save the name
-        Okular::Settings::setAnnotationsAuthor( userName );
+        Okular::Settings::setIdentityAuthor( userName );
         Okular::Settings::writeConfig();
     }
 }
@@ -694,8 +717,9 @@ QRect PageViewAnnotator::routeEvent( QMouseEvent * e, PageViewItem * item )
         {
             if ( !annotation ) continue;
 
-            annotation->creationDate = annotation->modifyDate = QDateTime::currentDateTime();
-            annotation->author = Okular::Settings::annotationsAuthor();
+            annotation->setCreationDate( QDateTime::currentDateTime() );
+            annotation->setModificationDate( QDateTime::currentDateTime() );
+            annotation->setAuthor( Okular::Settings::identityAuthor() );
             m_document->addPageAnnotation( m_lockedItem->pageNumber(), annotation );
         }
 
