@@ -24,11 +24,12 @@
 #include <qtoolbar.h>
 #include <kaction.h>
 #include <kactioncollection.h>
-#include <kinstance.h>
+#include <kglobalsettings.h>
 #include <klocale.h>
 #include <kiconloader.h>
 #include <kmessagebox.h>
 #include <kshortcut.h>
+#include <kdialog.h>
 
 // system includes
 #include <stdlib.h>
@@ -38,6 +39,8 @@
 #include "presentationwidget.h"
 #include "annotationtools.h"
 #include "pagepainter.h"
+#include "core/audioplayer.h"
+#include "core/document.h"
 #include "core/generator.h"
 #include "core/page.h"
 #include "core/link.h"
@@ -57,6 +60,30 @@ struct PresentationFrame
 };
 
 
+// a custom QToolBar that basically does not propagate the event if the widget
+// background is not automatically filled
+class PresentationToolBar : public QToolBar
+{
+    public:
+        PresentationToolBar( QWidget * parent = 0 )
+            : QToolBar( parent )
+        {}
+
+    protected:
+        void mousePressEvent( QMouseEvent * e )
+        {
+            QToolBar::mousePressEvent( e );
+            e->accept();
+        }
+
+        void mouseReleaseEvent( QMouseEvent * e )
+        {
+            QToolBar::mouseReleaseEvent( e );
+            e->accept();
+        }
+};
+
+
 PresentationWidget::PresentationWidget( QWidget * parent, Okular::Document * doc )
     : QDialog( parent, Qt::FramelessWindowHint ),
     m_pressedLink( 0 ), m_handCursor( false ), m_drawingEngine( 0 ), m_document( doc ),
@@ -66,10 +93,10 @@ PresentationWidget::PresentationWidget( QWidget * parent, Okular::Document * doc
     setAttribute( Qt::WA_DeleteOnClose );
     setAttribute( Qt::WA_OpaquePaintEvent );
     setObjectName( "presentationWidget" );
-    QString caption = doc->getMetaData( "DocumentTitle" ).toString();
+    QString caption = doc->metaData( "DocumentTitle" ).toString();
     if ( caption.trimmed().isEmpty() )
         caption = doc->currentDocument().fileName();
-    setWindowTitle( KInstance::makeStdCaption( caption ) );
+    setWindowTitle( KDialog::makeStandardCaption( caption ) );
 
     m_width = -1;
 
@@ -85,6 +112,9 @@ PresentationWidget::PresentationWidget( QWidget * parent, Okular::Document * doc
     m_overlayHideTimer = new QTimer( this );
     m_overlayHideTimer->setSingleShot( true );
     connect( m_overlayHideTimer, SIGNAL( timeout() ), this, SLOT( slotHideOverlay() ) );
+    m_nextPageTimer = new QTimer( this ); 
+//    m_nextPageTimer->setSingleShot( true );
+    connect( m_nextPageTimer, SIGNAL( timeout() ), this, SLOT( slotNextPage() ) ); 
 
     // handle cursor appearance as specified in configuration
     if ( Okular::Settings::slidesCursor() == Okular::Settings::EnumSlidesCursor::HiddenDelay )
@@ -102,6 +132,9 @@ PresentationWidget::PresentationWidget( QWidget * parent, Okular::Document * doc
 
 PresentationWidget::~PresentationWidget()
 {
+    // stop the audio playbacks
+    Okular::AudioPlayer::instance()->stopPlaybacks();
+
     // remove this widget from document observer
     m_document->removeObserver( this );
 
@@ -171,8 +204,7 @@ void PresentationWidget::notifyViewportChanged( bool /*smoothMove*/ )
     changePage( m_document->viewport().pageNumber );
 
     // auto advance to the next page if set
-    if ( Okular::Settings::slidesAdvance() )
-        QTimer::singleShot( Okular::Settings::slidesAdvanceTime() * 1000, this, SLOT( slotNextPage() ) );
+    startAutoChangeTimer();
 }
 
 void PresentationWidget::notifyPageChanged( int pageNumber, int changedFlags )
@@ -182,7 +214,7 @@ void PresentationWidget::notifyPageChanged( int pageNumber, int changedFlags )
         generatePage( changedFlags & DocumentObserver::Annotations );
 }
 
-bool PresentationWidget::canUnloadPixmap( int pageNumber )
+bool PresentationWidget::canUnloadPixmap( int pageNumber ) const
 {
     // can unload all pixmaps except for the currently visible one
     return pageNumber != m_frameIndex;
@@ -222,13 +254,16 @@ void PresentationWidget::keyPressEvent( QKeyEvent * e )
 {
     if (m_width == -1) return;
 
-    if ( e->key() == m_ac->action( "previous_page" )->shortcut().keyQt() || e->key() == Qt::Key_Left || e->key() == Qt::Key_Backspace || e->key() == Qt::Key_PageUp )
+#ifdef __GNUC__
+#warning FIX the usage of KActionCollection actions
+#endif
+    if ( /*e->key() == m_ac->action( "previous_page" )->shortcut().keyQt() || */e->key() == Qt::Key_Left || e->key() == Qt::Key_Backspace || e->key() == Qt::Key_PageUp )
         slotPrevPage();
-    else if ( e->key() == m_ac->action( "next_page" )->shortcut().keyQt() || e->key() == Qt::Key_Right || e->key() == Qt::Key_Space || e->key() == Qt::Key_PageDown )
+    else if ( /*e->key() == m_ac->action( "next_page" )->shortcut().keyQt() || */e->key() == Qt::Key_Right || e->key() == Qt::Key_Space || e->key() == Qt::Key_PageDown )
         slotNextPage();
-    else if ( e->key() == m_ac->action( "first_page" )->shortcut().keyQt() || e->key() == Qt::Key_Home )
+    else if ( /*e->key() == m_ac->action( "first_page" )->shortcut().keyQt() || */e->key() == Qt::Key_Home )
         slotFirstPage();
-    else if ( e->key() == m_ac->action( "last_page" )->shortcut().keyQt() || e->key() == Qt::Key_End )
+    else if ( /*e->key() == m_ac->action( "last_page" )->shortcut().keyQt() || */e->key() == Qt::Key_End )
         slotLastPage();
     else if ( e->key() == Qt::Key_Escape )
     {
@@ -248,14 +283,14 @@ void PresentationWidget::wheelEvent( QWheelEvent * e )
         if ( div > 3 )
             div = 3;
         while ( div-- )
-            slotPrevPage();
+            slotNextPage();
     }
     else if ( div < 0 )
     {
         if ( div < -3 )
             div = -3;
         while ( div++ )
-            slotNextPage();
+            slotPrevPage();
     }
 }
 
@@ -375,7 +410,7 @@ void PresentationWidget::paintEvent( QPaintEvent * pe )
         m_height = d.height();
 
         // create top toolbar
-        m_topBar = new QToolBar( this );
+        m_topBar = new PresentationToolBar( this );
         m_topBar->setObjectName( "presentationBar" );
         m_topBar->setIconSize( QSize( 32, 32 ) );
         m_topBar->addAction( KIcon( layoutDirection() == Qt::RightToLeft ? "1rightarrow" : "1leftarrow" ), i18n("Previous Page"), this, SLOT( slotPrevPage() ) );
@@ -384,7 +419,7 @@ void PresentationWidget::paintEvent( QPaintEvent * pe )
         sp.setHorizontalPolicy( QSizePolicy::Minimum );
         m_pagesEdit->setSizePolicy( sp );
         QFontMetrics fm( m_pagesEdit->font() );
-        m_pagesEdit->setMaximumWidth( fm.width( QString::number( m_document->pages() ) + "00" ) );
+        m_pagesEdit->setMaximumWidth( fm.width( QString::number( m_document->pages() ) ) * 2 );
         QIntValidator *validator = new QIntValidator( 1, m_document->pages(), m_pagesEdit );
         m_pagesEdit->setValidator( validator );
         m_topBar->addWidget( m_pagesEdit );
@@ -490,7 +525,7 @@ const Okular::Link * PresentationWidget::getLink( int x, int y, QRect * geometry
 
     // check if 1) there is an object and 2) it's a link
     QRect d = KGlobalSettings::desktopGeometry( const_cast< PresentationWidget * >( this ) );
-    const Okular::ObjectRect * object = page->getObjectRect( Okular::ObjectRect::Link, nx, ny, d.width(), d.height() );
+    const Okular::ObjectRect * object = page->objectRect( Okular::ObjectRect::Link, nx, ny, d.width(), d.height() );
     if ( !object )
         return 0;
 
@@ -502,7 +537,7 @@ const Okular::Link * PresentationWidget::getLink( int x, int y, QRect * geometry
     }
 
     // return the link pointer
-    return (Okular::Link *)object->pointer();
+    return (Okular::Link *)object->object();
 }
 
 void PresentationWidget::testCursorOnLink( int x, int y )
@@ -544,7 +579,6 @@ void PresentationWidget::changePage( int newPage )
     PresentationFrame * frame = m_frames[ m_frameIndex ];
     int pixW = frame->geometry.width();
     int pixH = frame->geometry.height();
-    int rot = frame->page->orientation();
 
     bool signalsBlocked = m_pagesEdit->signalsBlocked();
     m_pagesEdit->blockSignals( true );
@@ -558,11 +592,30 @@ void PresentationWidget::changePage( int newPage )
         // operation will take long: set busy cursor
         QApplication::setOverrideCursor( KCursor::workingCursor() );
         // request the pixmap
-        QLinkedList< Okular::PixmapRequest * > request;
-        request.push_back( new Okular::PixmapRequest( PRESENTATION_ID, m_frameIndex, pixW, pixH, rot, PRESENTATION_PRIO ) );
-        m_document->requestPixmaps( request );
+        QLinkedList< Okular::PixmapRequest * > requests;
+        requests.push_back( new Okular::PixmapRequest( PRESENTATION_ID, m_frameIndex, pixW, pixH, PRESENTATION_PRIO, false ) );
         // restore cursor
         QApplication::restoreOverrideCursor();
+        // ask for next and previous page if not in low memory usage setting
+        if (Okular::Settings::memoryLevel() != Okular::Settings::EnumMemoryLevel::Low && Okular::Settings::enableThreading()) {
+            if (newPage + 1 < (int)m_document->pages())
+            {
+                PresentationFrame *nextFrame = m_frames[ newPage + 1 ];
+                pixW = nextFrame->geometry.width();
+                pixH = nextFrame->geometry.height();
+                if ( !nextFrame->page->hasPixmap( PRESENTATION_ID, pixW, pixH ) )
+                    requests.push_back( new Okular::PixmapRequest( PRESENTATION_ID, newPage + 1, pixW, pixH, PRESENTATION_PRELOAD_PRIO, true ) );
+            }
+            if (newPage - 1 >= 0)
+            {
+                PresentationFrame *prevFrame = m_frames[ newPage - 1 ];
+                pixW = prevFrame->geometry.width();
+                pixH = prevFrame->geometry.height();
+                if ( !prevFrame->page->hasPixmap( PRESENTATION_ID, pixW, pixH ) )
+                    requests.push_back( new Okular::PixmapRequest( PRESENTATION_ID, newPage - 1, pixW, pixH, PRESENTATION_PRELOAD_PRIO, true ) );
+            }
+        }
+        m_document->requestPixmaps( requests );
     }
     else
     {
@@ -573,9 +626,20 @@ void PresentationWidget::changePage( int newPage )
     // set a new viewport in document if page number differs
     if ( m_frameIndex != -1 && m_frameIndex != m_document->viewport().pageNumber )
     {
+        // stop the audio playback, if any
+        Okular::AudioPlayer::instance()->stopPlaybacks();
+        // perform the page closing action, if any
+        if ( m_document->page( m_document->viewport().pageNumber )->pageAction( Okular::Page::Closing ) )
+            m_document->processLink( m_document->page( m_document->viewport().pageNumber )->pageAction( Okular::Page::Closing ) );
+
         // remove the drawing on the old page before switching
         clearDrawings();
         m_document->setViewportPage( m_frameIndex, PRESENTATION_ID );
+
+        // perform the page opening action, if any
+        if ( m_document->page( m_frameIndex)->pageAction( Okular::Page::Opening ) )
+            m_document->processLink( m_document->page( m_frameIndex )->pageAction( Okular::Page::Opening ) );
+
     }
 }
 
@@ -605,7 +669,7 @@ void PresentationWidget::generatePage( bool disableTransition )
     if ( !disableTransition )
     {
         const Okular::PageTransition * transition = m_frameIndex != -1 ?
-            m_frames[ m_frameIndex ]->page->getTransition() : 0;
+            m_frames[ m_frameIndex ]->page->transition() : 0;
         if ( transition )
             initTransition( transition );
         else {
@@ -855,6 +919,18 @@ QRect PresentationWidget::routeMouseDrawingEvent( QMouseEvent * e )
     return ret;
 }
 
+void PresentationWidget::startAutoChangeTimer()
+{
+    double pageDuration = m_frameIndex >= 0 && m_frameIndex < (int)m_frames.count() ? m_frames[ m_frameIndex ]->page->duration() : -1;
+    if ( Okular::Settings::slidesAdvance() || pageDuration > 0.0 )
+    {
+        double secs = pageDuration <= 0.0
+                   ? Okular::Settings::slidesAdvanceTime()
+                   : qMin<double>( pageDuration, Okular::Settings::slidesAdvanceTime() );
+        m_nextPageTimer->start( (int)( secs * 1000 ) );
+    }
+}
+
 
 
 void PresentationWidget::slotNextPage()
@@ -868,8 +944,7 @@ void PresentationWidget::slotNextPage()
         // go to next page
         changePage( m_frameIndex + 1 );
         // auto advance to the next page if set
-        if ( Okular::Settings::slidesAdvance() )
-            QTimer::singleShot( Okular::Settings::slidesAdvanceTime() * 1000, this, SLOT( slotNextPage() ) );
+        startAutoChangeTimer();
     }
     else
     {
@@ -895,8 +970,7 @@ void PresentationWidget::slotPrevPage()
         changePage( m_frameIndex - 1 );
 
         // auto advance to the next page if set
-        if ( Okular::Settings::slidesAdvance() )
-            QTimer::singleShot( Okular::Settings::slidesAdvanceTime() * 1000, this, SLOT( slotNextPage() ) );
+        startAutoChangeTimer();
     }
     else
     {
