@@ -48,7 +48,7 @@ class PresentationWidget : public QDialog, public Okular::DocumentObserver
         void notifySetup( const QVector< Okular::Page * > & pages, bool documentChanged );
         void notifyViewportChanged( bool smoothMove );
         void notifyPageChanged( int pageNumber, int changedFlags );
-        bool canUnloadPixmap( int pageNumber );
+        bool canUnloadPixmap( int pageNumber ) const;
 
         // create actions that interact with this widget
         void setupActions( KActionCollection * collection );
@@ -76,6 +76,7 @@ class PresentationWidget : public QDialog, public Okular::DocumentObserver
         const Okular::PageTransition defaultTransition() const;
         const Okular::PageTransition defaultTransition( int ) const;
         QRect routeMouseDrawingEvent( QMouseEvent * );
+        void startAutoChangeTimer();
 
         // cache stuff
         int m_width;
@@ -92,6 +93,7 @@ class PresentationWidget : public QDialog, public Okular::DocumentObserver
         // transition related
         QTimer * m_transitionTimer;
         QTimer * m_overlayHideTimer;
+        QTimer * m_nextPageTimer;
         int m_transitionDelay;
         int m_transitionMul;
         QList< QRect > m_transitionRects;
