@@ -8,49 +8,73 @@
  ***************************************************************************/
 
 // qt/kde includes
+#include <qdom.h>
 #include <qheaderview.h>
 #include <qlayout.h>
 #include <qstringlist.h>
-#include <qtoolbutton.h>
 #include <qtreewidget.h>
 #include <kicon.h>
 #include <klocale.h>
 #include <ktreewidgetsearchline.h>
 
 // local includes
+#include "pageitemdelegate.h"
 #include "toc.h"
 #include "core/document.h"
 #include "core/page.h"
-#include "settings.h"
 #include "core/link.h"
 
-// uncomment following to enable a 2nd column showing the page referred
-// by each tree entry note: PDF uses often references to viewports and
-// they're slow when converted to page number. drop the 2nd column idea.
-// to enable set TocPageColumn=true in [Nav Panel]
-
 class TOCItem : public QTreeWidgetItem
 {
     public:
-        TOCItem( QTreeWidget *parent, TOCItem *after, const QDomElement & e )
-            : QTreeWidgetItem( parent, after ), m_element( e )
+        TOCItem( QTreeWidget *parent, TOCItem *after, Okular::Document *document, const QDomElement & e )
+            : QTreeWidgetItem( parent, after )
         {
-            setText( 0, e.tagName() );
-            if ( Okular::Settings::tocPageColumn() && e.hasAttribute( "Page" ) )
-                setText( 1, e.attribute( "Page" ) );
+            init( document, e );
         }
 
-        TOCItem( QTreeWidgetItem *parent, TOCItem *after, const QDomElement & e )
-            : QTreeWidgetItem( parent, after ), m_element( e )
+        TOCItem( QTreeWidgetItem *parent, TOCItem *after, Okular::Document *document, const QDomElement & e )
+            : QTreeWidgetItem( parent, after )
         {
-            setText( 0, e.tagName() );
-            if ( Okular::Settings::tocPageColumn() && e.hasAttribute( "Page" ) )
-                setText( 1, e.attribute( "Page" ) );
+            init( document, e );
         }
 
-        const QDomElement & element() const
+        void init( Okular::Document *document, const QDomElement & e )
         {
-            return m_element;
+            // viewport loading
+            if ( e.hasAttribute( "Viewport" ) )
+            {
+                // if the node has a viewport, set it
+                m_viewport = Okular::DocumentViewport( e.attribute( "Viewport" ) );
+            }
+            else if ( e.hasAttribute( "ViewportName" ) )
+            {
+                // if the node references a viewport, get the reference and set it
+                const QString & page = e.attribute( "ViewportName" );
+                QString viewport = document->metaData( "NamedViewport", page ).toString();
+                if ( !viewport.isNull() )
+                    m_viewport = Okular::DocumentViewport( viewport );
+            }
+
+            if ( m_viewport.isValid() )
+            {
+                setData( 0, PageItemDelegate::PageRole, QString::number( m_viewport.pageNumber + 1 ) );
+                QString label = document->page( m_viewport.pageNumber )->label();
+                if ( !label.isEmpty() )
+                    setData( 0, PageItemDelegate::PageLabelRole, label );
+            }
+            m_extFileName = e.attribute( "ExternalFileName" );
+            setText( 0, e.tagName() );
+        }
+
+        QString externalFileName() const
+        {
+            return m_extFileName;
+        }
+
+        const Okular::DocumentViewport& viewport() const
+        {
+            return m_viewport;
         }
 
         void setCurrent( bool selected )
@@ -59,7 +83,8 @@ class TOCItem : public QTreeWidgetItem
         }
 
     private:
-        QDomElement m_element;
+        Okular::DocumentViewport m_viewport;
+        QString m_extFileName;
 };
 
 TOC::TOC(QWidget *parent, Okular::Document *document) : QWidget(parent), m_document(document), m_current(0), m_currentPage(-1)
@@ -78,14 +103,12 @@ TOC::TOC(QWidget *parent, Okular::Document *document) : QWidget(parent), m_docum
     m_treeView = new QTreeWidget( this );
     mainlay->addWidget( m_treeView );
     QStringList cols;
-    cols.append( i18n("Topic") );
-    if (Okular::Settings::tocPageColumn())
-        cols.append( i18n("Page") );
+    cols.append( "Topics" );
     m_treeView->setHeaderLabels( cols );
     m_treeView->setSortingEnabled( false );
     m_treeView->setRootIsDecorated( true );
     m_treeView->setAlternatingRowColors( true );
-    m_treeView->header()->setResizeMode( QHeaderView::Stretch );
+    m_treeView->setItemDelegate( new PageItemDelegate( m_treeView ) );
     m_treeView->header()->hide();
     m_treeView->setSelectionBehavior( QAbstractItemView::SelectRows );
     connect( m_treeView, SIGNAL( itemClicked( QTreeWidgetItem *, int ) ), this, SLOT( slotExecuted( QTreeWidgetItem * ) ) );
@@ -147,7 +170,7 @@ void TOC::notifyViewportChanged( bool /*smoothMove*/ )
     while ( (*it) && !m_current )
     {
         TOCItem *tmp = dynamic_cast<TOCItem*>( *it );
-        int p = tmp ? getViewport( tmp->element() ).pageNumber : -1;
+        int p = tmp ? tmp->viewport().pageNumber : -1;
         if ( p == newpage )
         {
             m_current = tmp;
@@ -159,6 +182,12 @@ void TOC::notifyViewportChanged( bool /*smoothMove*/ )
 }
 
 
+void TOC::reparseConfig()
+{
+    m_treeView->update();
+}
+
+
 void TOC::addChildren( const QDomNode & parentNode, QTreeWidgetItem * parentItem )
 {
     // keep track of the current listViewItem
@@ -171,9 +200,9 @@ void TOC::addChildren( const QDomNode & parentNode, QTreeWidgetItem * parentItem
 
         // insert the entry as top level (listview parented) or 2nd+ level
         if ( !parentItem )
-            currentItem = new TOCItem( m_treeView, currentItem, e );
+            currentItem = new TOCItem( m_treeView, currentItem, m_document, e );
         else
-            currentItem = new TOCItem( parentItem, currentItem, e );
+            currentItem = new TOCItem( parentItem, currentItem, m_document, e );
 
         // descend recursively and advance to the next node
         if ( e.hasChildNodes() )
@@ -188,36 +217,17 @@ void TOC::slotExecuted( QTreeWidgetItem *i )
     // that filters clicks on [+] that for a strange reason don't seem to be TOCItem*
     if (tocItem == NULL)
         return;
-    const QDomElement & e = tocItem->element();
 
-    QString externalFileName = e.attribute( "ExternalFileName" );
+    QString externalFileName = tocItem->externalFileName();
     if ( !externalFileName.isEmpty() )
     {
-        Okular::LinkGoto link( externalFileName, getViewport( e ) );
+        Okular::LinkGoto link( externalFileName, tocItem->viewport() );
         m_document->processLink( &link );
     }
     else
     {
-        m_document->setViewport( getViewport( e ) );
+        m_document->setViewport( tocItem->viewport() );
     }
 }
 
-Okular::DocumentViewport TOC::getViewport( const QDomElement &e ) const
-{
-    if ( e.hasAttribute( "Viewport" ) )
-    {
-        // if the node has a viewport, set it
-        return Okular::DocumentViewport( e.attribute( "Viewport" ) );
-    }
-    else if ( e.hasAttribute( "ViewportName" ) )
-    {
-        // if the node references a viewport, get the reference and set it
-        const QString & page = e.attribute( "ViewportName" );
-        QString viewport = m_document->getMetaData( "NamedViewport", page ).toString();
-        if ( !viewport.isNull() )
-            return Okular::DocumentViewport( viewport );
-    }
-    return Okular::DocumentViewport();
-}
-
 #include "toc.moc"
